/* ------------------------------------------------------------------------ *
 * SAS Institute, Inc.                                                      *
 *                                                                          *
 * Copyright (c) 2002-2007 SAS Institute, Inc.  All rights reserved.             *
  *                                                                          *
 * Contains functions needed by filter and rank dialog used for child       *
 * queries.                                                                 *
 *                                                                          *
 * Dependencies: citation.js, citation_button.js, citation_events.js,       *
 * citation_listTools.js, citation_msgFormatUtil.js, sas_treeview.js,       *
 * citation_relationalTimeConditionPanel.js,                                *
 * citation_olapTimeConditionPanel.js.                                      *                                                                         
 * ------------------------------------------------------------------------ */
wrsProvide("citation_filterRankCrosstab");
wrsRequire("citation", SCRIPT_DIR,  "citation_filterRankCrosstab");
wrsRequire("citation_button",  SCRIPT_DIR, "citation_filterRankCrosstab");
wrsRequire("citation_events",  SCRIPT_DIR, "citation_filterRankCrosstab");
wrsRequire("citation_listTools",  SCRIPT_DIR, "citation_filterRankCrosstab");
wrsRequire("citation_msgFormatUtil",  SCRIPT_DIR, "citation_filterRankCrosstab");
wrsRequire("sas_treeview",  SCRIPT_DIR, "citation_filterRankCrosstab");
wrsRequire("citation_olapTimeConditionPanel",  SCRIPT_DIR, "citation_filterRankCrosstab");

FilterRankDialog.CONDITION_TYPE_MEMBER = "CONDITION_TYPE_MEMBER";
FilterRankDialog.CONDITION_TYPE_MEMBER_PROPERTY = "categoryBasedMemberProperty";

//boolean that indicates whether OK to submit form. Set this flag immediately
//after form is submitted to prevent any subsequent submits while the submit is
//being processed
FilterRankDialog.prototype.isOkToSubmit = function()
{
    return this._isOkToSubmit;
}
FilterRankDialog.prototype.setOkToSubmit = function(isOkToSubmit)
{
    this._isOkToSubmit = isOkToSubmit;
}

//boolean that indicates whether the submit was actually triggered
//from a legal context. The value of this boolean is set programattically
//so that submits are not allowed when for example the user presses ENTER
//in a text field (e.g. the year field). There should be an onsubmit handler
//registered on the form that returns this false when this flag is not set.
FilterRankDialog.prototype.isLegalSubmitContext = function()
{
    return this._isLegalSubmitContext;
}
FilterRankDialog.prototype.setLegalSubmitContext = function(isLegal)
{
    this._isLegalSubmitContext = isLegal;
}
FilterRankDialog.prototype.getValueChooserManagerInstances = function()
{
	return this._valueChooserManagerInstances;
}
FilterRankDialog.prototype.setValueChooserManagerInstances = function(vcm)
{
	this._valueChooserManagerInstances = vcm;
}
FilterRankDialog.prototype.isOlapData = function()
{
    return this._isOlapData;
}
FilterRankDialog.prototype.setOlapData = function(isOlap)
{
    this._isOlapData = isOlap;
}
FilterRankDialog.prototype.getFilterAxisSelectionEntries = function()
{
    if (this._filterAxisSelectionEntries == null) {
        this._filterAxisSelectionEntries = new Array();
    }
    return this._filterAxisSelectionEntries;
}
FilterRankDialog.prototype.addAxisSelectionEntry = function(selectionEntries, selectionEntry)
{
    selectionEntries[selectionEntries.length] = selectionEntry;
}
FilterRankDialog.prototype.getRankingAxisSelectionEntries = function()
{
    if (this._rankingAxisSelectionEntries == null) {
        this._rankingAxisSelectionEntries = new Array();
    }
    return this._rankingAxisSelectionEntries;
}
FilterRankDialog.prototype.getHintText = function(hintKey)
{
    return ( this.getHintTextMap()[hintKey] );
}
FilterRankDialog.prototype.getHintTextMap = function()
{
    if(this._hintTextMap == null) { 
        this._hintTextMap = new Array();
    }
    return this._hintTextMap;
}
//returns true if a formatted comparison is allowed for the
//selected category data item currently being filtered
FilterRankDialog.prototype.isFormattedComparisonAllowed = function()
{
    return this._isFormattedComparisonAllowed
}
FilterRankDialog.prototype.setFormattedComparisonAllowed = function(isAllowed)
{
    this._isFormattedComparisonAllowed = isAllowed;
}
//returns true if an unformatted comparison is allowed for 
//the selected category data item currently being filtered
FilterRankDialog.prototype.isUnformattedComparisonAllowed = function()
{
    return this._isUnformattedComparisonAllowed
}
FilterRankDialog.prototype.setUnformattedComparisonAllowed = function(isAllowed)
{
    this._isUnformattedComparisonAllowed = isAllowed;
}
FilterRankDialog.prototype.isFormattedUnformattedOptionAvailable = function()
{
    return this._isFormattedUnformattedOptionAvailable;
}
FilterRankDialog.prototype.setFormattedUnformattedOptionAvailable = function(isAvailable)
{
    this._isFormattedUnformattedOptionAvailable = isAvailable;
}
FilterRankDialog.prototype.updateIgnoreCaseCheckBox = function()
{
	var show = ( this.getSelectedConditionType() == FilterRankDialog.CONDITION_TYPE_MEMBER &&
		!isMemberPropertyFilter());
	if (show == true) {
		show = (this.isFormattedUnformattedOptionAvailable()) ? true : false;
    }
         
	wrsShowElement( findDOM('ignoreCaseRow'), show);
}
FilterRankDialog.prototype.putFieldSet = function(fieldSetKey, fieldSet)
{
    this._fieldSetMap[fieldSetKey] = fieldSet;
}
FilterRankDialog.prototype.getFieldSet = function(fieldSetKey)
{
    return this._fieldSetMap[fieldSetKey];
}
FilterRankDialog.prototype.getForm = function()
{
    return this._masterFormObj;
}
FilterRankDialog.prototype.isConditionTypeCharged = function(conditionTypeAsStr)
{
    if (this._conditionTypeChargeStatuses == null){
        return false;
    }
    var status = this._conditionTypeChargeStatuses[conditionTypeAsStr];
    if (status == null) {
        return false;
    }
    return status;
}
FilterRankDialog.prototype.setConditionTypeChargeStatus = function(conditionTypeAsStr, isCharged)
{
    if (this._conditionTypeChargeStatuses == null) {
        this._conditionTypeChargeStatuses = new Array();
    }
    this._conditionTypeChargeStatuses[conditionTypeAsStr] = isCharged;
}
FilterRankDialog.prototype.getOlapTimeBasedConditionPanel = function()
{
    return this._olapTimeBasedConditionPanel;
}
FilterRankDialog.prototype.setOlapTimeBasedConditionPanel = function(panel)
{
    this._olapTimeBasedConditionPanel = panel;
}
// ------------------------------------------------------------------------
// Handler invoked when the user changes the selected category.
// ------------------------------------------------------------------------
FilterRankDialog.prototype.onSelectedCategoryChange = function(newDataItemId, idInputFieldName)
{
    if ( !this.isOkToSubmit() )
    {
        return;
    }
   
    var objForm = this.getForm();
    var activeContainer = this.getActiveCategoryFilterContainer();
    if (activeContainer != null)
    {
	    if ( !this.preFilterCommitValidation(activeContainer) )
	    {
	        return false;
	    }
	    if (typeof activeContainer.prepareForSubmit == "function") //accounts for null or undefined
	    {
	        activeContainer.prepareForSubmit(objForm);
	    }
	}

    //update form field that specifies what command APPLY is to perform
    objForm.elements["commandId"].value = FilterRankDialog.COMMAND_ID_CHANGE_SELECTED_CATEGORY;
    var selectedIdInput = objForm.elements[idInputFieldName];
    var prevSelectedId = selectedIdInput.value;
    selectedIdInput.value = newDataItemId;
    var prevSelectedTableRow = findDOM("tr_selectDataItem_"+prevSelectedId);
    var newSelectedTableRow = findDOM("tr_selectDataItem_"+newDataItemId);
    var prevSelectedTableRowCells = prevSelectedTableRow.cells;
    var newSelectedTableRowCells = newSelectedTableRow.cells;
    var selectedRowColClass = prevSelectedTableRowCells[0].className;
	this.setValueChooserManagerInstances(new Array());

    for( var i=0; i<prevSelectedTableRowCells.length; i++ )
    {
      prevSelectedTableRowCells[i].className = "defaultDataItemCell";
      newSelectedTableRowCells[i].className = selectedRowColClass;
    }

    var ogAction = objForm.action;
    objForm.action = this._applyActionHandlerPath;
    //NOTE:
    // 1) targeted action should persist any changes to prev selected item 
    //    & init form bean with filter information of new selected item
    //
    // 2) form-submission by itself should preserve state of other tab
    //    (measures tab) since there are no specialized controls on that
    //    tab.
    
    this.submitFilterAndRankDlgForm();
  
    objForm.action = ogAction; //reset back to original
}
  
FilterRankDialog.prototype.preFilterCommitValidation = function(activeContainer)
{
    //ensure that active container has valid info b4 submitting form
    var objForm = this.getForm();
    if (!activeContainer.validate) return true;
    var invalidFormElement = activeContainer.validate(objForm);
    if( invalidFormElement != null )
    {
    	if (activeContainer.validate.validationErrorMessage.length > 0) {
        	alert(activeContainer.validate.validationErrorMessage);
        }
        if (invalidFormElement.focus != null) {
            invalidFormElement.focus();
        }
        return false;
    }

    return true;
}
FilterRankDialog.prototype.scrollSelectedCategoryItemIntoView = function()
{
    var objForm = this.getForm();
    var selectedIndexInput = objForm.elements['newSelectedCategoryNodeIndex'];
    if (selectedIndexInput == null) {
        return;
    }
    var selectedDiTableRow = findDOM("tr_selectDataItem_"+selectedIndexInput.value);
    if( selectedDiTableRow != null ) { dojo.html.scrollIntoView(selectedDiTableRow); }
	var selectedDiTableCell = findDOM("tr_selectDataItemSpan_"+selectedIndexInput.value);
	var isCategoryTabVisible = (findDOM("tabBodyCategoryFilters").style.display != "none");
	if (isCategoryTabVisible && selectedDiTableCell != null ) { selectedDiTableCell.focus(); }
}
FilterRankDialog.prototype.getSelectedConditionType = function() 
{
    var objForm = this.getForm();
    var selectedConditionType = "";
    var radioGrp = objForm.elements["categoryConditionType"];
    if (!radioGrp) return selectedConditionType;
    for (var i=0; i<radioGrp.length; i++) {
    	if (radioGrp[i].checked) {
    		selectedConditionType = radioGrp[i].value;
    	 	if (selectedConditionType == FilterRankDialog.CONDITION_TYPE_MEMBER &&
    	 		isMemberPropertyFilter(objForm)) {
    	 		selectedConditionType = FilterRankDialog.CONDITION_TYPE_MEMBER_PROPERTY;
    	}
    }
    }
    return selectedConditionType;
}
// -----------------------------------------------------------------------------
// returns the active container that contains the form fields for constructing
// a filter of the type that corresponds to the form's currently selected filter
// type.
// -----------------------------------------------------------------------------
FilterRankDialog.prototype.getActiveCategoryFilterContainer = function()
{
    var activeContainerKey = this.getSelectedConditionType();
    retval = this.categoryConditionTypeContainers[activeContainerKey];    
    return retval;
}
FilterRankDialog.prototype.onCategoryConditionTypeChange = function()
{
    if (this.chargeSelectedConditionTypeIfNeeded()) {
        return;
    }
    this.updateCategoryConditionTypeContainersVisibility();
}
//returns true if condition type had to be charge as part of processing;
//false otherwise
FilterRankDialog.prototype.chargeSelectedConditionTypeIfNeeded = function()
{
    var objForm = this.getForm();
    var conditionTypeId = this.getSelectedConditionType();
    if (conditionTypeId == FilterRankDialog.CONDITION_TYPE_MEMBER_PROPERTY) return false;
    var isConditionTypeCharged = this.isConditionTypeCharged(conditionTypeId);
    if (isConditionTypeCharged) {
        return false;
    }
    
    //prepare previously charged condition type containers for 
    //submission so as to remember the settings on the the redraw and upon
    //a return to that filter type in the same dlg session.
    this.prepareChargedFilterTypesForSubmission(objForm);
        
    //update field that indicates the command to be performed ...
    objForm.elements["commandId"].value = FilterRankDialog.COMMAND_ID_CHANGE_FILTER_TYPE; 
  
    //update field that indicates the request is to be processed by APPLY ...
    var ogAction = objForm.action;
    objForm.action = this._applyActionHandlerPath;
        
    //submit form 
    this.submitFilterAndRankDlgForm();
    objForm.action = ogAction; //reset back to original       
        
    return true;
    
}
FilterRankDialog.prototype.onFormattedComparisonCheckboxChange = function()
{
    var activeContainer = this.getActiveCategoryFilterContainer();
    if (activeContainer != null 
        && activeContainer.processFormattedComparisonState != null)
    {
        activeContainer.processFormattedComparisonState();
    }
}
FilterRankDialog.prototype.chargeMemberSelectionBasedOnFormattedComparisonState = function(isFormattedComparisonRequest)
{
/* ----
TODO finish impl
-- */
    var objForm = this.getForm();
    var conditionTypeId = this.getSelectedConditionType();
    
    //prepare previously charged condition type containers for 
    //submission so as to remember the settings on the the redraw and upon
    //a return to that filter type in the same dlg session.
    this.prepareChargedFilterTypesForSubmission(objForm);
    //update field that indicates the command to be performed ...
    objForm.elements["commandId"].value = FilterRankDialog.COMMAND_ID_CHANGE_IS_FORMATTED_FILTER; 
  
    //update field that indicates the request is to be processed by APPLY ...
    var ogAction = objForm.action;
    objForm.action = this._applyActionHandlerPath;
        
    //submit form 
    this.submitFilterAndRankDlgForm();
    objForm.action = ogAction; //reset back to original       
        
    return true;
}
//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
// Submits the dialogs form and performs any post submit activities. All submits
// of the corresponding dialog should be done using this method. This means
// care must be taken to disable the default submit actions of certain fields
// (e.g. textEntrys) contained within the form.
//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
FilterRankDialog.prototype.submitFilterAndRankDlgForm = function()
{
    var objForm = this.getForm();
    this.setLegalSubmitContext(true); //all contexts that submit using this method are legal
    if (this.isOkToSubmit())
    {
        //submit the form, notifying any listeners of the pending event beforehand
        eventNotify(objForm,'submit',false);
        objForm.submit();
        this.afterFilterAndRankDlgSubmit()
    }
}
//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
// Should be invoked after submit is called. This function displays the
// wait cursor within the dialog and updates the form so that no subsequent form
// submits can occur.
//
// WARNING: It is essential that this be called after submit and not before.
// This is b/c this method may result in form fields being disabled (maybe
// indirectly - the parent container is disabled) and the field values
// will not be submitted if disabled prior to the submit call.
//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
FilterRankDialog.prototype.afterFilterAndRankDlgSubmit = function()
{
    var objForm = this.getForm();
    this.setOkToSubmit(false);
    //change to wait cursor since all submissions of this form result in a page reload.
    if (setDocumentCursorStyle != null) {
        setDocumentCursorStyle("wait");
    }

    //disable OK, CANCEL, HELP buttons which are outside the form
    //document.body.disabled = true;
    if( setButtonState != null ) {
        setButtonState(null /*event*/, "rankFilterSubmitButton", setButtonState.DISABLED );
        setButtonState(null /*event*/, "rankFilterCancelButton", setButtonState.DISABLED );
        setButtonState(null /*event*/, "helpButton", setButtonState.DISABLED );
    }

    //disable data item selection table. Some user agents might not honor this
    //so as added precaution set a flag that indicates subsequent submissions of
    //of form are not allowed. NOTE: use table not scrolling div. Wierd alignment
    //issues occur when disabling the div.
    //div id ="rankAndFilterDataItemSelectionContainer"
    var diSelectContainer = findDOM("rankAndFilterDataItemSelectionTableBody");
    if (diSelectContainer != null)
    {
       diSelectContainer.disabled = true;
    }

    setFormElementsEnabled(objForm, false);
}

FilterRankDialog.prototype.updateCategoryConditionTypeContainersVisibility = function()
{
    var activeContainerKey = this.getSelectedConditionType();
    _updateActiveContainerForSet(this.categoryConditionTypeContainers, this.categoryConditionTypeContainers[activeContainerKey]);
    wrsShowElement(findDOM("rankAndFilterMemberPropertyContainer"), 
		((activeContainerKey == FilterRankDialog.CONDITION_TYPE_MEMBER && this._isMemberPropertyAvailable) ||
    	activeContainerKey == FilterRankDialog.CONDITION_TYPE_MEMBER_PROPERTY));
}

FilterRankDialog.prototype.onDialogLoaded = function()
{
    this.scrollSelectedCategoryItemIntoView();
    
    this.updateCategoryConditionTypeContainersVisibility();
    updateFilterValueQualifierBasedControls();
    updateRankValueQualifierBasedControls();
    
    var olapTimePanel = this.getOlapTimeBasedConditionPanel();
    if (olapTimePanel != null)
    {
        olapTimePanel.initialize();
        olapTimePanel.activate();
    }
    else if (this._relationalTimeConditionPanel != null)
    {
        this._relationalTimeConditionPanel.initialize();
        this._relationalTimeConditionPanel.activate();
    }
    
    //ensure upper left corner is appropriately visible on page loads
    window.scroll(0, 0);
    this.enableLoadButtons();
}

FilterRankDialog.prototype.enableLoadButtons = function()
{
    var objForm = this.getForm();
    var formattedFilterCb = objForm.elements["formattedComparisonCheckbox"];
    if (!formattedFilterCb) {
       // if this isn't there, neither are the buttons
       return;
    }
    var valueTypeFormatted = formattedFilterCb.checked;
    if (valueTypeFormatted) 
    {
       if (this._needFormattedValueLoad && !this._dataItemTypeInOnly)
       {
           // enable formatted buttons
           if (findDOM('formattedValueSelectorForInExpression_loadValuesButton') != null)
			setButtonState(null, 'formattedValueSelectorForInExpression_loadValuesButton', setButtonState.ENABLED);
       }
       else
       {
          // disable formatted buttons
        	if (findDOM('formattedValueSelectorForInExpression_loadValuesButton') != null)
        	  setButtonState(null, 'formattedValueSelectorForInExpression_loadValuesButton', setButtonState.DISABLED);
       }
    }
    if (!valueTypeFormatted)
    { 
       if (this._needUnformattedValueLoad && !this._dataItemTypeInOnly)
       {
	      // enable unformatted buttons
          if (findDOM('unformattedValueSelectorForInExpression_loadValuesButton') != null)
        	 setButtonState(null, 'unformattedValueSelectorForInExpression_loadValuesButton', setButtonState.ENABLED);
    	}
       else
       {
    	   // disable unformatted buttons
           if (findDOM('unformattedValueSelectorForInExpression_loadValuesButton') != null)
        	 setButtonState(null, 'unformattedValueSelectorForInExpression_loadValuesButton', setButtonState.DISABLED);
       }
    }
}

FilterRankDialog.prototype.loadFilterValues = function()
{
    var objForm = this.getForm();
    var formattedFilterCb = objForm.elements["formattedComparisonCheckbox"];
    var valueTypeFormatted = formattedFilterCb.checked;
    if ((!valueTypeFormatted && g_filterRankDialog._unformattedMemberSelectionConditionCharged) ||
       (valueTypeFormatted && g_filterRankDialog._formattedMemberSelectionConditionCharged)) {
        return false;
    }
    
    //update field that indicates the command to be performed ...
    objForm.elements["commandId"].value = FilterRankDialog.COMMAND_ID_LOAD_FILTER_VALUES; 
    
    this.prepareChargedFilterTypesForSubmission(objForm);
         
    //update field that indicates the request is to be processed by APPLY ...
    var ogAction = objForm.action;
    objForm.action = this._applyActionHandlerPath;
        
    //submit form 
    this.submitFilterAndRankDlgForm();
    objForm.action = ogAction; //reset back to original       
        
    return true;
    
}
FilterRankDialog.prototype.loadSearchResults = function()
{
    var objForm = this.getForm();
    objForm.elements["commandId"].value = FilterRankDialog.COMMAND_ID_LOAD_SEARCH_RESULTS;
    this.prepareChargedFilterTypesForSubmission(objForm);
    
	if (objForm.elements["selectedCategoryNode.textToMatch"].value.length < 1) return false;	
    //update field that indicates the request is to be processed by APPLY ...
    var ogAction = objForm.action;
    objForm.action = this._applyActionHandlerPath;
        
    //submit form 
    this.submitFilterAndRankDlgForm(objForm);
    objForm.action = ogAction; //reset back to original       
        
    return true;
}
FilterRankDialog.prototype.prepareChargedFilterTypesForSubmission = function(objForm)
{
    var radioGrp = objForm.elements["categoryConditionType"];
    if (!radioGrp) return;
    for (var i=0; i<radioGrp.length; i++) {
        var conditionTypeId = radioGrp[i].value;
        var chargeStatus = this.isConditionTypeCharged(conditionTypeId);
        var commandValue = objForm.elements["commandId"].value;
        //alert("charge status for condition type [" + conditionTypeId + "] := " 
        //       + chargeStatus
        //       + "\ncommandValue := " + commandValue);
        if (chargeStatus 
            || commandValue == FilterRankDialog.COMMAND_ID_LOAD_SEARCH_RESULTS
        	|| commandValue == FilterRankDialog.COMMAND_ID_LOAD_FILTER_VALUES
        	) 
        { //as desired this check skips the requested condition type as well             
            var ctContainer = this.categoryConditionTypeContainers[conditionTypeId];
            if (ctContainer != null && typeof ctContainer.prepareForSubmit == "function") 
            {
                ctContainer.prepareForSubmit(objForm);
            }
        } 
    }
}

FilterRankDialog.prototype.submitAndScroll = function(actionUrl) 
{
    var objForm = this.getForm();
    onCommit_processMemberListSelections(); // Fixes 7610960892/S0949051
    var savedAction = objForm.action;
    objForm.action = actionUrl;
    this.prepareChargedFilterTypesForSubmission(objForm);
    this.submitFilterAndRankDlgForm(objForm);
    objForm.action = savedAction;
}

FilterRankDialog.prototype._unformattedMemberSelectionConditionCharged = false;
FilterRankDialog.prototype._formattedMemberSelectionConditionCharged = false;

FilterRankDialog.prototype.addMemberProperty = function(name, label, dataType)
{
	var memberProperty = new CategoryMemberProperty(name, label, dataType);
	this.categoryMemberProperties[this.categoryMemberProperties.length] = memberProperty;
}

FilterRankDialog.prototype.getDataTypeOfMemberProperty = function(index)
{
	if (index < 0 || index >= this.categoryMemberProperties.length) return 0; // undefined
	var memberProperty = this.categoryMemberProperties[index];
	return memberProperty.getDataType();
}

function FilterRankDialog() 
{
    this.setOlapData(false);
    this.setLegalSubmitContext(false);
    this.setOkToSubmit(true);
    this._isMeasureBasedFilterCreationEnabled = true;
    this._isMultiplePersistedMeasureConditions = false;
    this._reportMode = FilterRankDialog.REPORT_MODE_BUILDER;
    this._isUnformattedComparisonAllowed = false;
    this._isFormattedComparisonAllowed = false;  
    this._masterFormObj = null;  
    this._fieldSetMap = new Array();  
    
    this.categoryConditionTypeContainers = new Array();
    this.categoryMemberProperties = new Array();
	this._valueChooserManagerInstances = new Array();
}

FilterRankDialog.VALUE_QUALIFIER_ROWS                   = "VALUE_QUALIFIER_ROWS";
FilterRankDialog.VALUE_QUALIFIER_COLUMNS                = "VALUE_QUALIFIER_COLUMNS";
FilterRankDialog.COMMAND_ID_CHANGE_SELECTED_CATEGORY    = null; //TO BE SET IN CONSUMING JSP
FilterRankDialog.COMMAND_ID_CHANGE_IS_FORMATTED_FILTER  = null; //TO BE SET IN CONSUMING JSP
FilterRankDialog.MEMBER_TREE_NODE_ATTRIBUTE_RAW_VALUE   = "ATTRIBUTE_RAW_VALUE";

//report mode constants
FilterRankDialog.REPORT_MODE_BUILDER = 0;
FilterRankDialog.REPORT_MODE_VIEWER  = 1;

//Javascript object that is used to model the dialog and its information.
//Note:
// 
// 1) This object does not represent the window but instead a logical
//    representation of the dialog.
// 2) Supports at most one dialog per window
//
var g_filterRankDialog = new FilterRankDialog(); 

function CategoryMemberProperty(name, label, dataType)
{
	this._name = name; 
	this._label = label;
	this._dataType = dataType;
}

CategoryMemberProperty.prototype.getName = function() { return this._name; }

CategoryMemberProperty.prototype.getLabel = function() { return this._label; }

CategoryMemberProperty.prototype.getDataType = function() { return this._dataType; }

function preTabSelection(prevSelectedTab, targetSelectedTab)
{
    if (g_tabBodyContainers != null) 
    {
        //find tabBody that g_tabBodyContainers maps to the specified tab id
        var prevActiveTabBody = g_tabBodyContainers[prevSelectedTab.id]; //store active container so that we can show it as last step
	    if (prevActiveTabBody.validate != null)
	    {
	        var validationResult = prevActiveTabBody.validate();
	        if (validationResult != null) {
	            alert(validationResult);
	            return false;
	        }
	        return true;
	    }
	}
    
    //veto tab selection if leaving category tab and tab is not valid
    return true;

}

/* 
>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
  CATEGORY BASED FILTERING
>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
*/

function enableFormatCheckbox(enable)
{
    var formatCbLabel = findDOM("formattedComparisonCheckboxLabel");
    if (formatCbLabel == null)
        return;
    var formatCb = findDOM("formattedComparisonCheckbox");
    if (formatCb == null)
        return;
        
    formatCbLabel.className = enable ? "label" : "labeldisabled";
    formatCb.disabled = !enable;    
}

function isMemberPropertyFilter(objForm) {
	var memberPropertyCheckBox = findDOM("memberPropertyEnabled");
	if (memberPropertyCheckBox == null) return false;
	return memberPropertyCheckBox.checked;
}

// ------------------------------------------------------------------------
// Handler that is to process keypresses on items in list of available category.
// ------------------------------------------------------------------------
    function _processKeyPressForDataItemLabel(evt, delegateFunction, tableName, index)
    {
	    var evt = (evt) ? evt : ((window.event) ? window.event : null);
        var keyCode = ( evt.which != null ) ? evt.which : evt.keyCode;
	    // return key was pressed
	    if (keyCode === '\r'.charCodeAt(0))
	    {
	        if (delegateFunction != null) {
	            delegateFunction();
	        }
	    }
	    else {
		  var arrowUpKey = 38;
		  var arrowDownKey = 40;
		        			
		  if(keyCode == arrowUpKey || keyCode == arrowDownKey){
		     var table = findDOM(tableName);
		     var rows = table.rows;
		     var newIndex = index;
		     if (keyCode == arrowUpKey) {
		         newIndex = (index == 0) ? rows.length - 1 : index - 1;
		     }
		     if (keyCode == arrowDownKey) {
		        newIndex = (index == rows.length - 1) ? 0 : index + 1; 
		     }
		     rows[newIndex].onclick();
		     dojo.html.scrollIntoView(rows[newIndex]);
		  }
	    }
    }

function updateFormatCheckbox(show)
{
    var formatCbContainer = findDOM("container_formattedComparisonOptions")
    var containerVisible = formatCbContainer != null 
            && formatCbContainer.style.display != "none";
    if (containerVisible
            && g_filterRankDialog.isFormattedComparisonAllowed()
            && g_filterRankDialog.isUnformattedComparisonAllowed())
    {
        enableFormatCheckbox(show); 
    }
    g_filterRankDialog.updateIgnoreCaseCheckBox();
}
function onShowFilterTypeNoneControls()
{
	updateFormatCheckbox(false);
}
function onShowFilterTypeMemberPropertyControls()
{
	var memberProperty = findDOM("memberProperty");
	var selectedIndex = memberProperty.selectedIndex;
	updateFormatCheckbox(false);
}
function onShowValueSelectionContainerForMemberExpression()
{
	updateFormatCheckbox(true)
}

function processFormattedComparisonState_forDates()
{
    updateValueSelectorVisibilityForMemberExpression();
}

function processFormattedComparisonState_selectedValuesForMemberExpression()
{
    var objForm = g_filterRankDialog.getForm();

    var formattedComparisonCb = objForm.elements["selectedCategoryNode.formattedComparison"];
    if (formattedComparisonCb == null) {
        return;
    }

   	var valueTypeFormatted = formattedComparisonCb.checked;
	var unformattedSearchChoice = findDOM("unformattedValueSelectorForInExpression_searchChoice");
	var unformattedSearchValue = findDOM("unformattedValueSelectorForInExpression_searchValue");
	var formattedSearchChoice = findDOM("formattedValueSelectorForInExpression_searchChoice");
	var formattedSearchValue = findDOM("formattedValueSelectorForInExpression_searchValue");
	if (valueTypeFormatted &&
		formattedSearchChoice != null && formattedSearchValue != null) {
		  formattedSearchChoice.value = EditFilterForm_SEARCH_OPERATOR_CONTAINS;
		  formattedSearchValue.value = '';
	}
	else if (unformattedSearchChoice != null && unformattedSearchValue != null) {
		unformattedSearchChoice.value = EditFilterForm_SEARCH_OPERATOR_CONTAINS;
		unformattedSearchValue.value = '';
	}
    updateValueSelectorVisibilityForMemberExpression();
}

function updateValueSelectorVisibilityForMemberExpression()
{
    var objForm = g_filterRankDialog.getForm();
    var formattedComparisonCb = objForm.elements["selectedCategoryNode.formattedComparison"];
    if (formattedComparisonCb == null) {
        return;
    }
    
    //container_selectUnformattedValuesContainer
    //container_selectFormattedValuesContainer
    var unformattedValueSelectorContainer 
        = findDOM("container_selectUnformattedValuesContainer");
    var formattedValueSelectorContainer 
        = findDOM("container_selectFormattedValuesContainer");
    var unformattedDateValueSelectorContainer
    	= findDOM("container_categoryConditionType_timeBased");
   	var formattedComparison = formattedComparisonCb.checked;
   	
   	//check whether selector targeted for visibility has been charged
   	var selectorCharged = formattedComparison
   	    ? g_filterRankDialog._formattedMemberSelectionConditionCharged
   	    : g_filterRankDialog._unformattedMemberSelectionConditionCharged;
   	
   	if (selectorCharged)
   	{
   	    wrsShowElement(formattedValueSelectorContainer, formattedComparison);
   	    if (g_filterRankDialog._isRelationalDateOrTime) {
   	    	wrsShowElement(unformattedDateValueSelectorContainer, !formattedComparison);
   	    }
   	    else {
       		wrsShowElement(unformattedValueSelectorContainer, !formattedComparison);
        }
        g_filterRankDialog.enableLoadButtons();
    }
    else
    {
        //submit form requesting a change in the formatted checkbox with
        //the appropriate charging
        g_filterRankDialog.chargeMemberSelectionBasedOnFormattedComparisonState(formattedComparison);
    }
}

function validate_tabBodyCategoryFilters()
{
    var objForm = g_filterRankDialog._masterFormObj;
    var activeContainer = g_filterRankDialog.getActiveCategoryFilterContainer();
    if (activeContainer != null)
    {
	    var invalidFormElement = activeContainer.validate(objForm);
	    if( invalidFormElement != null )
	    {
	        return activeContainer.validate.validationErrorMessage;
	    }
	}
	
	return null; //validation succeeded
    
}

function validate_selectCategoryValues(objForm)
{
    var formattedFilterCb = objForm.elements["formattedComparisonCheckbox"];
    var valueTypeFormatted = formattedFilterCb.checked;
    
    var valueSelectionWidget = (valueTypeFormatted) ? objForm.selectedFormattedValuesWidgetForInExpression :
      	objForm.selectedUnformattedValuesWidgetForInExpression;
      	
    if (valueSelectionWidget.options.length < 1) {
        validate_selectCategoryValues.validationErrorMessage = 
            validate_selectCategoryValues.msgNoMembersSelected;
        
        return validate_selectCategoryValues.invalidRetval;
    }
    
    return null;
}

function validate_memberTreeSelections(objForm)
{
    var selectedMembers = eval('filterValueSelection_selectedValuesListBox');
    if (selectedMembers == null || selectedMembers.options.length < 1) {
        validate_memberTreeSelections.validationErrorMessage = 
            validate_memberTreeSelections.msgNoMembersSelected;
        
        return validate_memberTreeSelections.invalidRetval;
    }
    return null;
}

//define dummy NON-NULL object we can return to indicate that the selections are invalid 
validate_memberTreeSelections.invalidRetval = new Object();
//define dummy NON-NULL object we can return to indicate that the selections are invalid 
validate_selectCategoryValues.invalidRetval = new Object();

function onCommit_processMemberTreeSelections()
{
    //prepare unformatted values for submission if the selector exists
    prepareMemberTreeSelectionsForSubmit(false);
}

function prepareMemberTreeSelectionsForSubmit(isFormattedValueSelector)
{
    var objForm = g_filterRankDialog.getForm();
	var searchChoice = findDOM("filterValueSelection_searchChoice");
	var searchValue = findDOM("filterValueSelection_searchValue");
	if (searchChoice != null && searchValue != null) {
		objForm.elements["selectedCategoryNode.searchOperator"].value = searchChoice.value;
		objForm.elements["selectedCategoryNode.textToMatch"].value = searchValue.value;	
	}
    	
    var selectionComponent = eval('filterValueSelection_selectedValuesListBox');
    if (selectionComponent == null) {
       return;
    }
    if (selectionComponent.options.length < 1 &&
      	objForm.elements["commandId"].value != FilterRankDialog.COMMAND_ID_LOAD_SEARCH_RESULTS) {
      	return;
    }
    
    var memberFieldCache = g_filterRankDialog._memberFieldCache;
    var memberLabelFieldCache = g_filterRankDialog._memberLabelFieldCache;
    var memberParentPathFieldCache = g_filterRankDialog._memberParentPathFieldCache;
    if (memberFieldCache == null)
    {
        memberFieldCache = new Array();
        g_filterRankDialog._memberFieldCache = memberFieldCache;
        
        memberLabelFieldCache = new Array();
        g_filterRankDialog._memberLabelFieldCache = memberLabelFieldCache;
        
        memberParentPathFieldCache = new Array();
        g_filterRankDialog._memberParentPathFieldCache = memberParentPathFieldCache;
    }
    
    var fieldNameMemberUniqueName = "selectedCategoryNode.memberConditionAbstraction.members";
    detachFieldsFromForm(objForm, memberFieldCache);
    var fieldNameMemberLabel = "selectedCategoryNode.memberConditionAbstraction.memberLabels";
    detachFieldsFromForm(objForm, memberLabelFieldCache);
    var fieldNameMemberParentPath = "selectedCategoryNode.memberConditionAbstraction.memberParentPaths";
    detachFieldsFromForm(objForm, memberParentPathFieldCache);
        
    for( var i = 0; i<selectionComponent.options.length; i++ )
    {
        var currOption = selectionComponent.options[i];

        //NOTE: The name of all hidden fields should match the correspond beanProperty
        var memberUniqueNameField   = newHiddenField(fieldNameMemberUniqueName, currOption.value);
        objForm.appendChild( memberUniqueNameField );
        memberFieldCache[ memberFieldCache.length ] = memberUniqueNameField;
        
        var memberLabelField   = newHiddenField(fieldNameMemberLabel, getMemberLabel(currOption.text));
        objForm.appendChild( memberLabelField );
        memberLabelFieldCache[ memberLabelFieldCache.length ] = memberLabelField;
        
        var memberParentPathField   = newHiddenField(fieldNameMemberParentPath,getMemberPath(currOption.text));
        objForm.appendChild( memberParentPathField );
        memberParentPathFieldCache[ memberParentPathFieldCache.length ] = memberParentPathField;

    }//for( var i = 0; i<selectedMemberNodes.length
    showDebugMsgs('processMemberTreeSelections' , 'EXITING processMemberTreeSelections (values selected: ' + selectionComponent.options.length + ')');
    
}

function newHiddenField(fieldName, fieldValue)
{
    var field   = document.createElement('input');
    field.type  = 'hidden';
    field.name  = fieldName;
    field.value = fieldValue;
    return field;
}

function getMemberLabel(optionText)
{
	var index = optionText.indexOf('(');
	if (index < 0) {
		return optionText;
	}
	else {
	 	return optionText.substring(0, index-1);
	}
}

function getMemberPath(optionText)
{
	var index = optionText.indexOf('(');
	if (index < 0) {
		return '';
	}
	else {
	 	return optionText.substring(index+1, optionText.length-1);
	}
}

//calling this routine empties the array if not already
function detachFieldsFromForm(objForm, fieldArray)
{
    if (fieldArray != null && fieldArray.length > 0) {
        for( var j=fieldArray.length-1; j>=0; j-- ) {
            objForm.removeChild( fieldArray[j] );
        }
        fieldArray.length = 0; //empty the array
    }
}

function onCommit_processMemberListSelections()
{
    var objForm = g_filterRankDialog.getForm();
    _prepareMatchValueSettingsForSubmission(objForm, false);
    _prepareMatchValueSettingsForSubmission(objForm, true);
    var formattedComparisonCb = objForm.elements["selectedCategoryNode.formattedComparison"];
    if (formattedComparisonCb == null) {
        return;
    }

   	var valueTypeFormatted = formattedComparisonCb.checked;
   	if (valueTypeFormatted) {
    	var formattedSearchChoice = findDOM("formattedValueSelectorForInExpression_searchChoice");
    	var formattedSearchValue = findDOM("formattedValueSelectorForInExpression_searchValue");
    	if (formattedSearchChoice != null && formattedSearchValue != null) {
    		objForm.elements["selectedCategoryNode.searchOperator"].value = formattedSearchChoice.value;
    		objForm.elements["selectedCategoryNode.textToMatch"].value = formattedSearchValue.value;	
    	}
   	}
   	else {
    	var unformattedSearchChoice = findDOM("unformattedValueSelectorForInExpression_searchChoice");
    	var unformattedSearchValue = findDOM("unformattedValueSelectorForInExpression_searchValue");
		if (unformattedSearchChoice != null && unformattedSearchValue != null) {
			objForm.elements["selectedCategoryNode.searchOperator"].value = unformattedSearchChoice.value;
			objForm.elements["selectedCategoryNode.textToMatch"].value = unformattedSearchValue.value;
		}
   	}
    
}// end prepareForFormSubmit_matchValues() method definition

function _prepareMatchValueSettingsForSubmission(objForm, isFormattedValueSettings)
{
    // alert("preparing " + (isFormattedValueSettings?"formatted":"unformatted") + " values");
    var valueFieldsName = null;
    var valueSelectionWidget = null; //may still be null after attempted assignment
    
    if (isFormattedValueSettings)
    {
        valueFieldsName = "selectedCategoryNode.formattedSelectedMemberConditionAbstraction.members";
        valueSelectionWidget = objForm.selectedFormattedValuesWidgetForInExpression;
    }
    else
    {
        valueFieldsName = "selectedCategoryNode.memberConditionAbstraction.members";
        valueSelectionWidget = objForm.selectedUnformattedValuesWidgetForInExpression;
    }
    
    var prevMatchValueFields = objForm.elements[valueFieldsName];
    if( prevMatchValueFields != null )
    {
        if (prevMatchValueFields.length > 0) {
            var prevItemCount = prevMatchValueFields.length;
            for (var n=0; i<prevItemCount; i++) {
                var prevField = prevMatchValueFields[n];
                // alert("Removing field from document --> name [" + this.idField.name + "] with value { " + this.idField.value + " }") ;
                objForm.removeChild(prevField);
            }
        }
    }
    
    //no-op if no widget for selection present in ui (the option of filtering on both formatted
    //and unformatted values may not be an option or desired)
	if (valueSelectionWidget === undefined) {
		return;
	}
    if( valueSelectionWidget != null )
    {
        var valueObjs = valueSelectionWidget.options;
        var valueCount = valueObjs.length;
        for( var i=0; i<valueCount; i++ )
        {
            var valueObj = valueObjs[i];
            // alert("adding value index := " + valueObj.value + "; valueLabel := " + valueObj.text );
            var valueField = document.createElement('input');
            valueField.type = 'hidden';
            valueField.name = valueFieldsName;
            valueField.value = valueObj.value;
            objForm.appendChild(valueField);
        }//for
    }//if( valueSelectionWidget != null )
}// end _prepareMatchValueSettingsForSubmission() method definition

function setSelectedSearchChoice(selectId, selectedValueId) {
	var select = findDOM(selectId);
	var selectedValue = findDOM(selectedValueId).value;
	var nOptions = select.length;
	var selectedIndex = 0; // default to first one
	for (var i=0; i<select.length; i++) {
		var option = select.options[i];
		if (option.value == selectedValue) {
			selectedIndex = i;
			break;
		}
	}
	select.selectedIndex = selectedIndex;
}

function updateMemberPropertyDropDownControl() {
	g_filterRankDialog.onCategoryConditionTypeChange();	
	var objForm = g_filterRankDialog.getForm();
	var isMemberProperty = isMemberPropertyFilter(objForm);
	findDOM("memberProperty").disabled = !isMemberProperty;
    if (isMemberProperty) {
    	var memberPropertyIndex = objForm.elements["memberProperty"].selectedIndex;
    	var dataType = g_filterRankDialog.getDataTypeOfMemberProperty(memberPropertyIndex);
    	if (dataType == 1) {
    		wrsShowElement(findDOM("memberPropertyStringDiv"), true);
    		wrsShowElement(findDOM("memberPropertyNumericDiv"), false);
    		wrsShowElement(findDOM("memberPropertyBooleanDiv"), false);
    		wrsShowElement(findDOM("memberPropertyUndefinedDiv"), false);
    	}
    	else if (dataType == 2) {
    		wrsShowElement(findDOM("memberPropertyStringDiv"), false);
    		wrsShowElement(findDOM("memberPropertyNumericDiv"), true);
    		wrsShowElement(findDOM("memberPropertyBooleanDiv"), false);
    		wrsShowElement(findDOM("memberPropertyUndefinedDiv"), false);
    	}
    	else if (dataType == 3) {
    		wrsShowElement(findDOM("memberPropertyStringDiv"), false);
    		wrsShowElement(findDOM("memberPropertyNumericDiv"), false);
    		wrsShowElement(findDOM("memberPropertyBooleanDiv"), true);
    		wrsShowElement(findDOM("memberPropertyUndefinedDiv"), false);
    	}
    	else {
    		wrsShowElement(findDOM("memberPropertyStringDiv"), false);
    		wrsShowElement(findDOM("memberPropertyNumericDiv"), false);
    		wrsShowElement(findDOM("memberPropertyBooleanDiv"), false);
    		wrsShowElement(findDOM("memberPropertyUndefinedDiv"), true);
    	}
    }
}

// ************************** OLAP TIME BASED CONDITION ROUTINES ************************
function defineOlapTimeBasedConditionFieldSets()
{
    // XXXXXXXXXXXXXXXXXXXXXXXXXXXXX Boundary point related field sets XXXXXXXXXXXXXXXXXXXXXXXXXXXX 
    var timeRangeStartBoundaryFieldSet = new OlapTimeRangeBoundaryFieldSet(
        "timeRangeStartingPointFirstPeriodRadio",
        "timeRangeStartingPointFirstPeriodRadioLabel",
        "timeRangeStartingPointExplicitMemberRadio", 
        "timeRangeStartingPointExplicitMemberRadioLabel",
        "startingTimeMemberTree",
        "explicitMemberTreeArea_startingTime",
        "explicitMemberTreeContainer_startingTime",
        "explicitMemberTreeDisabledContainer_startingTime",
        "rtStartingTimeFieldSetLabel");
    timeRangeStartBoundaryFieldSet.setId("timeRangeStartBoundaryFieldSet");
    
    var timeRangeEndBoundaryFieldSet = new OlapTimeRangeBoundaryFieldSet(
        "timeRangeEndingPointLastPeriodRadio",
        "timeRangeEndingPointLastPeriodRadioLabel",
        "timeRangeEndingPointExplicitMemberRadio", 
        "timeRangeEndingPointExplicitMemberRadioLabel",
        "endingTimeMemberTree",
        "explicitMemberTreeArea_endingTime",
        "explicitMemberTreeContainer_endingTime",
        "explicitMemberTreeDisabledContainer_endingTime",
        "rtEndingTimeFieldSetLabel");
    timeRangeEndBoundaryFieldSet.setId("timeRangeEndBoundaryFieldSet");    
        
    // XXXXXXXXXXXXXXXXXXXXXXXXXXXXX N periods related field sets XXXXXXXXXXXXXXXXXXXXXXXXXXXXX     
    var nPeriodsShiftForwardOptionFieldSet = new OlapTimeNPeriodsShiftDirectionFieldSet(
        timeRangeStartBoundaryFieldSet.getId());
    nPeriodsShiftForwardOptionFieldSet.setId("rtNPeriodsShiftDirectionForwardRadio");
    
    var nPeriodsShiftBackwardsOptionFieldSet = new OlapTimeNPeriodsShiftDirectionFieldSet(
        timeRangeEndBoundaryFieldSet.getId());
    nPeriodsShiftBackwardsOptionFieldSet.setId("rtNPeriodsShiftDirectionBackwardsRadio");    
    
    var nPeriodsConditionFieldSet = new OlapTimeNPeriodsFieldSet(
        "rtNPeriodsConditionPeriodMultiplier",
        "rtNPeriodsShiftDirectionOptionContainer",
        "selectedCategoryNode.timeConditionForm.relativeNPeriodsForm.shiftDirection",
        "rtNPeriodsShiftBackwardsMemberTypeField",
        "rtNPeriodsShiftBackwardsMemberIdentifierField",
        "rtNPeriodsShiftForwardMemberTypeField",
        "rtNPeriodsShiftForwardMemberIdentifierField");
                
    nPeriodsConditionFieldSet.setId("rtConditionTypeNPeriodsRadio");
    nPeriodsConditionFieldSet.setShiftForwardFieldSet(nPeriodsShiftForwardOptionFieldSet);
    nPeriodsConditionFieldSet.setShiftBackwardsFieldSet(nPeriodsShiftBackwardsOptionFieldSet);
    
    // XXXXXXXXXXXXXXXXXXXXXXXXXXXXX Explicit Period range related field sets XXXXXXXXXXXXXXXXXXXXXXXXXXXX 
    var periodRangeFieldSet = new OlapTimeRangeFieldSet(
        "periodRangeExplicitStartMemberIdentifierTypeField",
        "periodRangeExplicitStartMemberIdentifierField",
        "periodRangeExplicitEndMemberIdentifierTypeField",
        "periodRangeExplicitEndMemberIdentifierField");
    periodRangeFieldSet.setRangeStartFieldSet(timeRangeStartBoundaryFieldSet);
    periodRangeFieldSet.setRangeEndFieldSet(timeRangeEndBoundaryFieldSet);
    periodRangeFieldSet.setId("rtConditionTypePeriodRangeRadio");
    
    // XXXXXXXXXXXXXXXXXXXXXXXXXXXXX Setup the panel with all the field sets defined above XXXXXXXXXXXXXXXXXXXXXxx
    var olapTimePanel = new OlapTimeBasedConditionPanel(
        g_filterRankDialog, 
        "rtPeriodTypeDropdown",
        "selectedCategoryNode.timeConditionForm.conditionType", //conditionTypeRadioNames,
        nPeriodsConditionFieldSet,
        timeRangeStartBoundaryFieldSet,
        timeRangeEndBoundaryFieldSet,
        periodRangeFieldSet
        );
    g_filterRankDialog.setOlapTimeBasedConditionPanel(olapTimePanel);
}

function onOlapTimeBasedConditionPeriodTypeChange()
{
    var timeConditionPanel = g_filterRankDialog.getOlapTimeBasedConditionPanel();
    timeConditionPanel.onPeriodTypeChange();    
}

//Processes 'onchange' events for each condition type radio
function onOlapTimeBasedConditionTypeChange(targetRadioId)
{
    var timeConditionPanel = g_filterRankDialog.getOlapTimeBasedConditionPanel();
    timeConditionPanel.onConditionTypeChange(targetRadioId);
}

//Processes 'onchange' events for eachshift direction radio
function onOlapTimeBasedNPeriodsShiftDirectionChange(targetRadioId)
{
    var timeConditionPanel = g_filterRankDialog.getOlapTimeBasedConditionPanel();
    timeConditionPanel.onNPeriodsShiftDirectionChange(targetRadioId);
}

function onOlapTimeRangeBoundaryPointIdentifierTypeChange(boundaryPointFieldSetKey)
{
    var timeConditionPanel = g_filterRankDialog.getOlapTimeBasedConditionPanel();
    timeConditionPanel.onRangeBoundaryPointIdentifierTypeChange(boundaryPointFieldSetKey);
}

function olapTimeBasedCondition_prepareForSubmit()
{
    var timeConditionPanel = g_filterRankDialog.getOlapTimeBasedConditionPanel();
    timeConditionPanel.prepareForSubmit();
}

// ************************** _END OLAP TIME BASED CONDITION ROUTINES ************************

/* 
>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
  MEASURE BASED FILTERING
>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
*/

function OperatorKeys() {};

//NOTE: JSP SHOULD OVERRIDE THE VALUES OF THESE CONSTANTS IF BACKEND VALUES
//NEED TO DIFFER
OperatorKeys.EQUAL                 = "OPERATOR_EQUAL";
OperatorKeys.NOT_EQUAL             = "OPERATOR_NOT_EQUAL";     
OperatorKeys.GREATER_THAN          = "OPERATOR_GREATER_THAN";
OperatorKeys.GREATER_THAN_OR_EQUAL = "OPERATOR_GREATER_THAN_OR_EQUAL";
OperatorKeys.LESS_THAN             = "OPERATOR_LESS_THAN";
OperatorKeys.LESS_THAN_OR_EQUAL    = "OPERATOR_LESS_THAN_OR_EQUAL";
OperatorKeys.BETWEEN               = "OPERATOR_BETWEEN";
OperatorKeys.NOT_BETWEEN           = "OPERATOR_NOT_BETWEEN";

function validate_tabBodyMeasureFilters()
{
    var objForm = g_filterRankDialog._masterFormObj;
    var activeContainer = getActiveMeasureConditionTypeContainer(objForm);  
    if (activeContainer != null && activeContainer.validate != null)
    {
    	var invalidFormElement = activeContainer.validate(objForm);
	    if( invalidFormElement != null )
	    {
	        return activeContainer.validate.validationErrorMessage;
	    }
    }
	return null; //validation succeeded
}

function validate_measureBasedFilter(objForm)
{
    var operatorSelectionField = objForm.elements["measureBasedFilterAbstraction.operator"];
    if (operatorSelectionField == null)
        return null;
        
    var selectedOperatorOption = operatorSelectionField.options[operatorSelectionField.selectedIndex];
    if (OperatorKeys.BETWEEN == selectedOperatorOption.value 
        || OperatorKeys.NOT_BETWEEN == selectedOperatorOption.value) 
    {
        //verify min and max nonempty
        var minField = objForm.elements["measureBasedFilterAbstraction.minValue"]
        if (wrs_filterRankUtil._isEmptyOrWhitespace(minField.value)) {
            validate_measureBasedFilter.validationErrorMessage = validate_measureBasedFilter.msgNoMinValueSpecified;
            return validate_measureBasedFilter.invalidRetval;
        }
        
        var maxField = objForm.elements["measureBasedFilterAbstraction.maxValue"];
        if (wrs_filterRankUtil._isEmptyOrWhitespace(maxField.value)) {
            validate_measureBasedFilter.validationErrorMessage = validate_measureBasedFilter.msgNoMaxValueSpecified;
            return validate_measureBasedFilter.invalidRetval;
        }
    }
    else
    {
        var compValueField = objForm.elements["measureBasedFilterAbstraction.value"];
        if (wrs_filterRankUtil._isEmptyOrWhitespace(compValueField.value)) {
            validate_measureBasedFilter.validationErrorMessage = validate_measureBasedFilter.msgNoComparisonValueSpecified;
            return validate_measureBasedFilter.invalidRetval;
        }
    }
        
    return null;
}
//define dummy NON-NULL objec we can return to indicate that the selections are invalid 
validate_measureBasedFilter.invalidRetval = new Object();

function validate_measureBasedRank(objForm)
{
    // 1) acquire refs to needed form ctrls
    var isTopRanking = findDOM("rankingOrder_top").checked;
    var rankAmountField = isTopRanking ? findDOM("rankingOrder_top_amount")    : findDOM("rankingOrder_bottom_amount");
    var isPercentField  = isTopRanking ? findDOM("rankingOrder_top_isPercent") : findDOM("rankingOrder_bottom_isPercent");
	
    // 2) determine if form ctrl values are valid
    var validationCode = validateRankingAmountForField(rankAmountField, isPercentField.checked);
    if ( validationCode != validateRankingAmount.VALID )
    {
        retval = rankAmountField;
        switch (validationCode) {
            case validateRankingAmount.NO_VALUE_SPECIFIED:
                validate_measureBasedRank.validationErrorMessage = validate_measureBasedRank.MSG_NO_VALUE_SPECIFIED != null ? validate_measureBasedRank.MSG_NO_VALUE_SPECIFIED : null;
                break;
            case validateRankingAmount.ILLEGAL_VALUE_SPECIFIED:
                validate_measureBasedRank.validationErrorMessage = validate_measureBasedRank.MSG_ILLEGAL_VALUE_SPECIFIED != null ? validate_measureBasedRank.MSG_ILLEGAL_VALUE_SPECIFIED : null
                break;
        }
        if( validate_measureBasedRank.validationErrorMessage == null )
        {
            validate_measureBasedRank.validationErrorMessage = "Please enter a valid rank value.";
        }
        return validate_measureBasedRank.invalidRetval;
    }
     
    return null;
}
//define dummy NON-NULL objec we can return to indicate that the selections are invalid 
validate_measureBasedRank.invalidRetval = new Object();


//
//
// @param targetActiveContainer object in containerSet that is to be made
//        active as a result of this call. Null may be specified for this 
//        to hide all containers in the set.
function _updateActiveContainerForSet(containerSet, targetActiveContainer)
{
    if (containerSet == null) {
        alert("ERROR: _updateActiveContainerForSet() invoked with no defined context containers.")
        return;
    }
    
    for (var containerKey in containerSet)
    {
        var container = containerSet[containerKey];
        if (container != null && container != targetActiveContainer) 
        {
            wrsShowElement(container, false);
            if (container.onHide != null) {
                container.onHide();    
            }
        }
    }
    
    if (targetActiveContainer != null) 
    {
        wrsShowElement(targetActiveContainer, true);
        if (targetActiveContainer.onShow != null) {
            targetActiveContainer.onShow();
        }
    }    
}

function onMeasureConditionTypeChange() 
{
    updateMeasureConditionTypeContainersVisibility();
}

function updateMeasureConditionTypeContainersVisibility()
{
    var objForm = g_filterRankDialog.getForm();
    var activeContainer = getActiveMeasureConditionTypeContainer(objForm);  
    _updateActiveContainerForSet(g_measureConditionTypeContainers, activeContainer);

}

function getActiveMeasureConditionTypeContainer(objForm)
{
    var activeContainerKey = null;
    var conditionTypeRadios = objForm.elements["measureBasedConditionType"];
    var radioCount = conditionTypeRadios.length;
    for (var i=0; i<radioCount; i++)
    {
        var radio = conditionTypeRadios[i];
        if (radio.checked) {
            activeContainerKey = radio.id;
            break;
        }
    }
    
    return g_measureConditionTypeContainers[activeContainerKey];
}

function onMeasureBasedFilterOperatorChange(operatorSelectionCtrlId) 
{
    updateMeasureBasedFilterOperatorCtrlGroupsVisibility(findDOM(operatorSelectionCtrlId));
}

function updateMeasureBasedFilterOperatorCtrlGroupsVisibility(operatorSelectionCtrl)
{
    var selectedOperatorOption = operatorSelectionCtrl.options[operatorSelectionCtrl.selectedIndex];
    var newActiveCtrlGroup = g_measureBasedFilterOperatorCtrlGroupMap[selectedOperatorOption.value];
    
    _updateActiveContainerForSet( g_measureBasedFilterOperatorCtrlGroupMap,
        newActiveCtrlGroup );
}

function onFilterValueQualifierChange()
{
    updateFilterValueQualifierBasedControls();
}

function updateFilterValueQualifierBasedControls()
{
    var objForm = g_filterRankDialog.getForm();
    var vqSelectionControl = objForm.elements["measureBasedFilterAbstraction.valueQualifier"];
    if (vqSelectionControl == null) {
        return; //measure tab must not be accessible
    }
    
    if (vqSelectionControl.selectedIndex < 0) {
        return; //no entries available for value qualifier (rare but possible - scatter plots)
    }
    var selectedVqOption = vqSelectionControl.options[vqSelectionControl.selectedIndex];
    var selectedVqValue = selectedVqOption.value;
 
    //1) update hint text for filter field set based on selected value qualifier
    var hintText = "";
    if (FilterRankDialog.VALUE_QUALIFIER_ROWS == selectedVqValue) {
        hintText = g_filterRankDialog.getHintText("rankAndFilter.measure.based.condition.hint.text.column.filter.txt");
    } else if (FilterRankDialog.VALUE_QUALIFIER_COLUMNS == selectedVqValue) {
        hintText = g_filterRankDialog.getHintText("rankAndFilter.measure.based.condition.hint.text.row.filter.txt");
    } else {
        hintText = g_filterRankDialog.getHintText("rankAndFilter.measure.based.condition.hint.text.filter.default.txt");
    }
    var hintContainer = findDOM("filterRankMeasureConditionTypeHintTextContainer_filter");
    hintContainer.firstChild.nodeValue = hintText;
    
    //2) update accessibility of "between" filter operators based on selected value qualifier
    var operatorSelectionControl = objForm.elements["measureBasedFilterAbstraction.operator"];
    var operatorOptions = operatorSelectionControl.options;
    
    if ( selectedVqValue == FilterRankDialog.VALUE_QUALIFIER_ROWS
         || selectedVqValue == FilterRankDialog.VALUE_QUALIFIER_COLUMNS)
    {
        //ensure BETWEEN operator is not surfaced
        if (g_filterRankDialog.cachedMeausureBasedBetweenOperator == null) 
        {
             var ogSelectedOperatorIndex = operatorSelectionControl.selectedIndex;
             var ogSelectedOperator = operatorOptions[ogSelectedOperatorIndex];
             var operatorCountB4Rmv = operatorOptions.length;
             var resetSelectedIndex = false;
             for (var operatorIndex=operatorCountB4Rmv-1; operatorIndex>=0; operatorIndex--)
             {
                 var operatorOption = operatorOptions[operatorIndex];
                 if (OperatorKeys.BETWEEN == operatorOption.value
                     || OperatorKeys.NOT_BETWEEN == operatorOption.value)
                 {
                     if (ogSelectedOperatorIndex == operatorIndex) {
                         //update flags so that they indicate that selection 
                         //should be reset but to first available operator
                         //since original selection will no longer be available
                         ogSelectedOperator = null; 
                         resetSelectedIndex = true;
                     } else if (ogSelectedOperatorIndex > operatorIndex) {
                         resetSelectedIndex = true;
                     }
                     
                     if (OperatorKeys.BETWEEN == operatorOption.value) {
                         g_filterRankDialog.cachedMeausureBasedBetweenOperator = operatorOption;
                     } else {
                         g_filterRankDialog.cachedMeausureBasedNotBetweenOperator = operatorOption;
                     }
                     operatorOptions[operatorIndex] = null;
                 }
             } //for (var operatorIndex=operatorCountB4Rmv-1;
             
             if (resetSelectedIndex) 
             {
                 operatorSelectionControl.defaultSelected = -1;
                 if (ogSelectedOperator == null)
                 {
                     operatorSelectionControl.selectedIndex = 0;
                 } 
                 else
                 {
                     //preserve og selection in the event the operator selected appeared after the BETWEEN operators
                     //Should never fall here given that BETWEEN operators currently appear last in list
                     for (var oIndex=0; oIndex<operatorOptions.length; oIndex++)
                     {
                         if (ogSelectedOperator == operatorOptions[oIndex])
                         {
                             operatorSelectionControl.selectedIndex = oIndex;
                             break;
                         }
                     }
                 } 
                 updateMeasureBasedFilterOperatorCtrlGroupsVisibility(operatorSelectionControl);
             }
        }//if (g_filterRankDialog.cachedMeausureBasedBetweenOperator == null) 
        
        //ensure that category selection controls are available for axis specification
        updateAxisSelectionEntriesVisibility(
            g_filterRankDialog.getFilterAxisSelectionEntries(),
            true //visible
            );
    }
    else
    {
        //ensure BETWEEN operator is surfaced
        if (g_filterRankDialog.cachedMeausureBasedBetweenOperator != null)
        {
            //need to resurface the options
            
            //cached the index since addition of items to options array causes selection to
            //to unwantedly change in IE
            var selectedIndexPreserved = operatorSelectionControl.selectedIndex; 
            operatorOptions[operatorOptions.length] = g_filterRankDialog.cachedMeausureBasedBetweenOperator;
            operatorOptions[operatorOptions.length] = g_filterRankDialog.cachedMeausureBasedNotBetweenOperator;
            g_filterRankDialog.cachedMeausureBasedBetweenOperator = null;
            g_filterRankDialog.cachedMeausureBasedNotBetweenOperator = null;
            operatorSelectionControl.selectedIndex = selectedIndexPreserved;
        } 
        
        //ensure that category selection controls are NOT available for axis specification
        updateAxisSelectionEntriesVisibility(
            g_filterRankDialog.getFilterAxisSelectionEntries(),
            false //visible
            );
    }
    
    //force ie to redraw selection ctrl; defeats ie bug
    //where empty item is left in list if one of items 
    //removed was outside the scrolled area at the time 
    //first selection was made
    wrsShowElement(vqSelectionControl, true);
}

function updateAxisSelectionEntriesVisibility(axisSelectionEntries, visible)
{
    for (var aseIndex=0; aseIndex<axisSelectionEntries.length; aseIndex++)
    {
        var axisSelectionEntry = axisSelectionEntries[aseIndex];
        if (!axisSelectionEntry.isMeasureEntry())
        {
            var entryCtrlContainers = axisSelectionEntry.getControlContainers();
            for (var containerIndex=0; containerIndex<entryCtrlContainers.length; containerIndex++)
            {
                var containerObj = entryCtrlContainers[containerIndex];
                wrsShowElement(containerObj, visible);
            }

        }
    }
}

function onRankValueQualifierChange()
{
    updateRankValueQualifierBasedControls();
}

function updateRankValueQualifierBasedControls()
{
    var objForm = g_filterRankDialog.getForm();
    var vqSelectionControl = objForm.elements["measureBasedRankAbstraction.valueQualifier"];
    if (vqSelectionControl == null) {
        return; //measure tab must not be accessible
    }
    if (vqSelectionControl.selectedIndex < 0) {
        return; //no entries available for value qualifier (rare but possible - scatter plots)
    }
    var selectedVqOption = vqSelectionControl.options[vqSelectionControl.selectedIndex];
    var selectedVqValue = selectedVqOption.value;
    
    //1) 
    enableRankingControls();
    
    //2) ensure that category selection controls are NOT available for axis specification
    
    var nonMeasureSelectionEntriesVisible = FilterRankDialog.VALUE_QUALIFIER_ROWS == selectedVqValue
        || FilterRankDialog.VALUE_QUALIFIER_COLUMNS == selectedVqValue;
        
    updateAxisSelectionEntriesVisibility(
            g_filterRankDialog.getRankingAxisSelectionEntries(),
            nonMeasureSelectionEntriesVisible //visible
            );
    
    //3) update hint text for filter field set based on selected value qualifier
    var hintText = "";
    if (FilterRankDialog.VALUE_QUALIFIER_ROWS == selectedVqValue) {
        hintText = g_filterRankDialog.getHintText("rankAndFilter.measure.based.condition.hint.text.column.rank.txt");
    } else if (FilterRankDialog.VALUE_QUALIFIER_COLUMNS == selectedVqValue) {
        hintText = g_filterRankDialog.getHintText("rankAndFilter.measure.based.condition.hint.text.row.rank.txt");
    } else {
        hintText = g_filterRankDialog.getHintText("rankAndFilter.measure.based.condition.hint.text.rank.default.txt");
    }
    
    var hintContainer = findDOM("filterRankMeasureConditionTypeHintTextContainer_rank");
    hintContainer.firstChild.nodeValue = hintText;
    
}

function enableRankingControls()
{
    enableRankingControlsBasedOnDefaultStatus();
    enableRankingControlsBasedOnPercentStatus(null);
}

function onRankingPercentFieldChange(percentFieldChangedId)
{
    enableRankingControlsBasedOnPercentStatus(percentFieldChangedId)
}

function enableRankingControlsBasedOnDefaultStatus()
{
	// default disabled/enabled status
    var topRankField = findDOM("rankingOrder_top"); 
   	findDOM('rankingOrder_top_amount').disabled = (topRankField.checked) ? false : true;
   	findDOM('rankingOrder_bottom_amount').disabled = (topRankField.checked) ? true : false;
   	findDOM('rankingOrder_top_isPercent').disabled = (topRankField.checked) ? false : true;
   	findDOM('rankingOrder_bottom_isPercent').disabled = (topRankField.checked) ? true : false;
   	findDOM('rankingOrder_top_isExcludeTies').disabled = (topRankField.checked) ? false : true;
   	findDOM('rankingOrder_bottom_isExcludeTies').disabled = (topRankField.checked) ? true : false;

   	findDOM('rankingOrder_top_isPercentLabel').className = (topRankField.checked) ? 'label' : 'labeldisabled';
   	findDOM('rankingOrder_bottom_isPercentLabel').className = (topRankField.checked) ? 'labeldisabled' : 'label';
   	findDOM('rankingOrder_top_isExcludeTiesLabel').className = (topRankField.checked) ? 'label' : 'labeldisabled';
   	findDOM('rankingOrder_bottom_isExcludeTiesLabel').className = (topRankField.checked) ? 'labeldisabled' : 'label';
   	
   	if (!g_isPercentageRankAllowed) {
	   	findDOM('rankingOrder_top_isPercent').disabled = true;
	   	findDOM('rankingOrder_bottom_isPercent').disabled = true;
	   	findDOM('rankingOrder_top_isPercentLabel').className = 'labeldisabled';
	   	findDOM('rankingOrder_bottom_isPercentLabel').className = 'labeldisabled';
   	}

}

//
// @param percentFieldId (optional) id of percent field for which controls are to be updated
//        based on its status. If null, then action will be taken based on the id of the selected 
//        percentField radio
function enableRankingControlsBasedOnPercentStatus(percentFieldId)
{
    // 1) acquire refs to needed form ctrls and define metadata needed to perform op
    var topRankField = findDOM("rankingOrder_top"); 
    var rankPercentField = null;
    if (percentFieldId != null) {
        rankPercentField = findDOM(percentFieldId);
    } else { //get checked order radio
        rankPercentField = topRankField.checked ? findDOM("rankingOrder_top_isPercent") : findDOM("rankingOrder_bottom_isPercent");
    }

    // special cases
    var isTopRankingField = (rankPercentField.id == "rankingOrder_top_isPercent");
    var isPercentField  = isTopRankingField ? findDOM("rankingOrder_top_isPercent") : findDOM("rankingOrder_bottom_isPercent");
    var excludeTiesField = isTopRankingField ? findDOM("rankingOrder_top_isExcludeTies") : findDOM("rankingOrder_bottom_isExcludeTies");
    var excludeTiesFieldLabel = isTopRankingField ? findDOM("rankingOrder_top_isExcludeTiesLabel") : findDOM("rankingOrder_bottom_isExcludeTiesLabel");

    // 2) enable/disable excludeTies field appropriately
    var excludeTiesEnabled = !isPercentField.checked; 
    if (isPercentField.checked) { // S0283992
    	excludeTiesField.checked = true;
    }
    
    excludeTiesField.disabled = !excludeTiesEnabled;
    excludeTiesFieldLabel.disabled = excludeTiesField.disabled;
    excludeTiesField.className = excludeTiesField.disabled ? "labeldisabled" : "label";    
}

function onRemoveMeasureFiltersCBClick()
{
    //update state of hidden field based on checked state of checkbox
    var objForm = g_filterRankDialog.getForm();
    var removeMeasureConditionsCb = objForm.elements["removeMeasureFiltersCb"];
    var removeMeasureConditionsOkField = objForm.elements["removeMeasureConditionsOk"]
    removeMeasureConditionsOkField.value = removeMeasureConditionsCb.checked
        ? "true" : "false";
}

function onRemoveMeasureFiltersButtonClick()
{
    var objForm = g_filterRankDialog.getForm();
    
    // 1) update form field that tells backend it is ok to remove any
    //    persisted measure conditions
    var removeMeasureConditionsOkField = objForm.elements["removeMeasureConditionsOk"]
    removeMeasureConditionsOkField.value = "true";
    
    if (g_filterRankDialog._isMeasureBasedFilterCreationEnabled)
    { 
        //UPON removal filter creation should be allowed
        
        //a) hide fieldset that displays persisted measure conditions
        //b) show controls associated with selected condition type radio 
        
        //accounts for a) & b) 
        updateMeasureConditionTypeContainersVisibility();  
        
        //c) enable measure condition type radios
        var conditionTypeRadios = objForm.elements["measureBasedConditionType"];
        var radioCount = conditionTypeRadios.length;
        for (var i=0; i<radioCount; i++) {
            var radio = conditionTypeRadios[i];
            var radioLabel = findDOM("labelFor_"+radio.id);
            if (radioLabel != null) {
                radioLabel.className = "label";
            }
            radio.disabled = false;
        }
    }
    else 
    {
        //=> Upon removal, cannot create any new measure filters 
        
        //NO ADDITIONAL ACTION REQUIRED
    }

}

// ------------------------------------------------------------
// Class that
// ------------------------------------------------------------
AxisSpecificationEntry.prototype.getControlContainers = function()
{
    if (this._ctrlContainers == null) {
        this._ctrlContainers = new Array();
    }
    return this._ctrlContainers;
}
AxisSpecificationEntry.prototype.getControlContainerCount = function()
{
    if (this._ctrlContainers == null) {
        return 0;
    }
    return this._ctrlContainers.length;
}
AxisSpecificationEntry.prototype.isMeasureEntry = function()
{
    return this._isMeasureEntry;
}
AxisSpecificationEntry.prototype.setMeasureEntry = function(isMeasureEntry)
{
    this._isMeasureEntry = isMeasureEntry;
}
function AxisSpecificationEntry(isMeasureEntry)
{
   this._ctrlContainer = null;   
   this._isMeasureEntry = isMeasureEntry == null ? false : isMeasureEntry;
}



/*
>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
 Commit logic 
>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
*/

function onFilterRankFormTagSubmit(event)
{
    //alert("vetoing submit := " + !g_filterRankDialog.isLegalSubmitContext());
    //veto the submit if not a legal form submit
    if (!g_filterRankDialog.isLegalSubmitContext())
    {
        if(WRSEventUtil.preventDefaultBehavior != null)
        {
            WRSEventUtil.preventDefaultBehavior(event);
        }
    }
    return g_filterRankDialog.isLegalSubmitContext();
}

// ------------------------------------------------------------------------
// Handler invoked when the user submits the form by pressing the ok
// button
// ------------------------------------------------------------------------
function onCommitRankFilter(formName, cmdKeyInputFieldName, commitCmdId)
{
    var objForm = g_filterRankDialog.getForm();
    var activeCategoryContainer = g_filterRankDialog.getActiveCategoryFilterContainer();
    
    if (activeCategoryContainer != null)
    {
        if ( !g_filterRankDialog.preFilterCommitValidation(activeCategoryContainer) )
        {
            return;
        }
        
        if (typeof activeCategoryContainer.prepareForSubmit == "function") //accounts for null or undefined
        {
            activeCategoryContainer.prepareForSubmit(objForm);
        }
    }
    
    var activeMeasureContainer = getActiveMeasureConditionTypeContainer(objForm); 
    if (activeMeasureContainer != null)
    {
        if ( !g_filterRankDialog.preFilterCommitValidation(activeMeasureContainer) )
        {
            return;
        }
    }

    //update the value of field that tells the mid-tier what cmd to execute
    var cmdKeyInputField = objForm.elements[cmdKeyInputFieldName];
    if( cmdKeyInputField != null )
    {
        cmdKeyInputField.value = commitCmdId;
    }
    g_filterRankDialog.submitFilterAndRankDlgForm();
}

function setFormElementsEnabled(objForm, enabled)
{
    var formEls = objForm.elements;
    var formElCount = formEls.length;
    for( var i=0; i<formElCount; i++ )
    {
        formEls[i].disabled = !enabled;
    }
}

//=================================================================
// GENERAL DOM UTILITY FUNCTIONS
//=================================================================

//-----------------------------------------------------------------
// Calculates and returns the number of pixels the given document
// element is from the left of current frame.
//-----------------------------------------------------------------
function cwCalcEffectiveOffsetLeft(docObj)
{
	var retval = 0;
	var positionedParent = docObj;
    while (positionedParent != null)
    {
    	//alert("positioned Parent type := " + positionedParent.tagName);
        retval += positionedParent.offsetLeft;
        //alert("adding := " + positionedParent.offsetLeft);
        positionedParent = positionedParent.offsetParent;
    }
    return retval;
}

function cwCalcEffectiveOffsetTop(docObj)
{
	var retval = 0;
	var positionedParent = docObj;
    while (positionedParent != null)
    {
        retval += positionedParent.offsetTop;
        positionedParent = positionedParent.offsetParent;
    }
    return retval;
}

// +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// CURRENTLY UNUSED SCRIPTS copied from V1 F&R dialog
// +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

// ------------------------------------------------------------------------
// Handler invoked when the user changes the order of the ranking using
// either the "top" or "bottom" labeled radio
// ------------------------------------------------------------------------
function onRankingOrderChange(topRankingRadioId, bottomRankingRadioId)
{
    var topRankingControls = onRankingOrderChange.topRankingControls;
    var bottomRankingControls = onRankingOrderChange.bottomRankingControls;
    var topRankingRadio = findDOM(topRankingRadioId);
    var bottomRankingRadio = findDOM(bottomRankingRadioId);

    var disableTopControls = !topRankingRadio.checked;
    for( var i=0; i<topRankingControls.length; i++ )
    {   //for now safe to assume topRankingControls.length == bottomRankingControls.length
        var topRankCtrl = topRankingControls[i];
        var bottomRankCtrl = bottomRankingControls[i]
        if (topRankCtrl.enabledStateModifiable != null && topRankCtrl.enabledStateModifiable) {
            topRankCtrl.disabled = disableTopControls;
        }
        if (bottomRankCtrl.enabledStateModifiable != null && bottomRankCtrl.enabledStateModifiable) {
          bottomRankCtrl.disabled = !disableTopControls;
        }
    }
}

function validateRankingAmountForField(rankAmountField, isPercent)
{
    var rankAmountValue = rankAmountField.value;
    rankAmountValue = (rankAmountValue.length == 0) ? null : window.parseInt(rankAmountValue, 10 /*base-10*/);
    return validateRankingAmount(rankAmountValue, isPercent);
}

function validateRankingAmount(rankAmount, isPercent)
{
    var retval = validateRankingAmount.VALID;
    //verify a positive base-10 # && (< 100 if isPercent)
    if ( rankAmount == null )
    {
        retval = validateRankingAmount.NO_VALUE_SPECIFIED;
    }
    else if ( ( window.isNaN != null && window.isNaN(rankAmount) )
        || (rankAmount <= 0)
        || (isPercent && rankAmount > 100 ) )
    {
       retval = validateRankingAmount.ILLEGAL_VALUE_SPECIFIED;
    }
    return retval;
}
//Possible return values for validateRankingAmount
validateRankingAmount.VALID = 0;
validateRankingAmount.NO_VALUE_SPECIFIED = 1;
validateRankingAmount.ILLEGAL_VALUE_SPECIFIED = 2;


/* ************************************************************************
  Methods used to validate group boxes on form submission
************************************************************************* */

function validate_filterNone(objForm)
{
    return null;
}

function validate_filterMemberProperties(objForm)
{
    var invalidField = null;
    var valueField = null;
    
	var objForm = g_filterRankDialog.getForm();
	var isMemberProperty = isMemberPropertyFilter(objForm);
	findDOM("memberProperty").disabled = !isMemberProperty;
    if (isMemberProperty) {
    	var memberPropertyIndex = objForm.elements["memberProperty"].selectedIndex;
    	var dataType = g_filterRankDialog.getDataTypeOfMemberProperty(memberPropertyIndex);
    	if (dataType == 1) {
    		valueField = findDOM("memberPropertyString_textEntry");
    	}
    	else if (dataType == 2) {
    		valueField = findDOM("memberPropertyNumeric_textEntry");
    	}
    }

    if (valueField != null)
    {   // validate measure value
        if( wrs_filterRankUtil._isEmptyOrWhitespace(valueField.value) )
        {
            invalidField = valueField;
            validate_filterMemberProperties.validationErrorMessage = validate_filterMemberProperties.msgNoValueEntered;
        }
    }

    return invalidField;
}

function validate_comparisonFilter(objForm)
{
    var invalidField = null;
    var valueField = objForm.elements["filterDefinitionForm.value"];

    if (valueField != null)
    {   // validate measure value
        if( wrs_filterRankUtil._isEmptyOrWhitespace(valueField.value) )
        {
            invalidField = valueField;
            if( validate_comparisonFilter.validationErrorMessage == null )
            {
                validate_comparisonFilter.validationErrorMessage = "A value must be entered!";
            }
        }
    }
    else
    {
       var operatorSelector = findDOM("filterRankDialogForm_timeCondition_operatorSelector");
       if (operatorSelector == null) return invalidField;
       
        //validate date fields if necessary
       var selectedOperatorOption = operatorSelector.options[operatorSelector.selectedIndex];
	    if (OperatorKeys.BETWEEN == selectedOperatorOption.value 
	        || OperatorKeys.NOT_BETWEEN == selectedOperatorOption.value) 
	    {
		   var startDateRadioFixed = findDOM("filterRankDialogForm_timeCondition_between_explicitValueOption_min");
		   if (startDateRadioFixed != null && startDateRadioFixed.checked) {
		       var startDateObj = sas_getJavaScriptObject('relationalTimeConditionForm_startDate_explicitDate_datePicker');
			   if (startDateObj != null) {
				   if (startDateObj.enabled && startDateObj.validate() == false) {
				      invalidField = startDateObj.comboComponent;
			          if( validate_comparisonFilter.validationErrorMessage == null ) {
			            validate_comparisonFilter.validationErrorMessage = "";
			          }
			          return invalidField;
				    }
			   }
		   }
		   
		   var endDateRadioFixed = findDOM("filterRankDialogForm_timeCondition_between_explicitValueOption_max");
		   if (endDateRadioFixed != null && endDateRadioFixed.checked) {
		       var endDateObj = sas_getJavaScriptObject('relationalTimeConditionForm_endDate_explicitDate_datePicker');
			   if (endDateObj != null) {
			   	  if (endDateObj.enabled && endDateObj.validate() == false) {
				      invalidField = endDateObj.comboComponent;
			          if( validate_comparisonFilter.validationErrorMessage == null ) {
			            validate_comparisonFilter.validationErrorMessage = "";
			          }
			      }
			   }
		   }
	    }
	    else 
	    {
	       var explicitDateRadioFixed = findDOM("filterRankDialogForm_timeCondition_scExplicitValueOption");
	       if (explicitDateRadioFixed != null && explicitDateRadioFixed.checked) {
		       var explicitDateObj = sas_getJavaScriptObject('relationalTimeConditionForm_dateValue_explicitDate_datePicker');
			   if (explicitDateObj != null) {
				   if (explicitDateObj.enabled && explicitDateObj.validate() == false) {
				      invalidField = explicitDateObj.comboComponent;
			          if( validate_comparisonFilter.validationErrorMessage == null ) {
			            validate_comparisonFilter.validationErrorMessage = "";
			          }
			          return invalidField;
				   }
			   }
		   }
	    }
	   
   }

    return invalidField;
}

/*
XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
  Methods used to validate form field values on edit
XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
 */

function wrs_filterRankUtil()
{

}

wrs_filterRankUtil.whiteSpaceRegExpTest = new RegExp("\\s", "g");

wrs_filterRankUtil._isEmptyOrWhitespace = function(strTest)
{
    //current impl doesn't account for string of whitespace
    var retval = ( strTest == null || strTest.length == 0 );

    if( !retval )
    {   //nonempty so test for all whitespace
        var matchResult = strTest.match(wrs_filterRankUtil.whiteSpaceRegExpTest);
        var isWhiteSpaceOnly = ( matchResult != null && matchResult.length == strTest.length );
        retval = isWhiteSpaceOnly;
    }
    return retval;
}

