/* ------------------------------------------------------------------------ *
 * SAS Institute, Inc.                                                      *
 *                                                                          *
 * Copyright (c) 2002-2007 SAS Institute, Inc.  All rights reserved.             *
 *                                                                          *
 * Contains functions needed by filter and rank dialog used for child       *
 * queries.                                                                 *
 *                                                                          *
 * Dependencies: citation.js, citation_button.js, citation_events.js,       *
 * citation_listTools.js, citation_msgFormatUtil.js, citation_fieldSet.js   *
 * ------------------------------------------------------------------------ */
wrsProvide("citation_filterRankList");
wrsRequire("citation", SCRIPT_DIR,  "citation_filterRankList");

//NOTE: JSP SHOULD OVERRIDE THE VALUES OF THESE CONSTANTS IF BACKEND VALUES
//NEED TO DIFFER
FilterRankDialog.OPERATOR_EQUAL                 = "OPERATOR_EQUAL";
FilterRankDialog.OPERATOR_NOT_EQUAL             = "OPERATOR_NOT_EQUAL";     
FilterRankDialog.OPERATOR_GREATER_THAN          = "OPERATOR_GREATER_THAN";
FilterRankDialog.OPERATOR_GREATER_THAN_OR_EQUAL = "OPERATOR_GREATER_THAN_OR_EQUAL";
FilterRankDialog.OPERATOR_LESS_THAN             = "OPERATOR_LESS_THAN";
FilterRankDialog.OPERATOR_LESS_THAN_OR_EQUAL    = "OPERATOR_LESS_THAN_OR_EQUAL";
FilterRankDialog.OPERATOR_BETWEEN               = "OPERATOR_BETWEEN";
FilterRankDialog.OPERATOR_NOT_BETWEEN           = "OPERATOR_NOT_BETWEEN";
FilterRankDialog.OPERATOR_MISSING               = "OPERATOR_MISSING";
FilterRankDialog.OPERATOR_NOT_MISSING           = "OPERATOR_NOT_MISSING";
FilterRankDialog.OPERATOR_MONTH_TO_DATE         = "OPERATOR_MONTH_TO_DATE";
FilterRankDialog.OPERATOR_YEAR_TO_DATE          = "OPERATOR_YEAR_TO_DATE";
FilterRankDialog.NONE_PANEL		= "none_panel";
FilterRankDialog.IN_PANEL			= "in_panel";
FilterRankDialog.MEASURE_DATE_PANEL = "measure_date_panel";
FilterRankDialog.RANK_PANEL		= "rank_panel";
FilterRankDialog.FILTER_NONE              = 1;
FilterRankDialog.FILTER_FILTER			  = 2;
FilterRankDialog.FILTER_RANK			    = 4;

//ALL TO BE AUTHORITATIVELY SET IN CONSUMING JSP
FilterRankDialog.COMMAND_ID_CHANGE_DATA_ITEM_SELECTION = "CMD_CHANGE_SELECTED_DATA_ITEM";
FilterRankDialog.COMMAND_ID_CHANGE_FILTER_TYPE         = "CMD_CHANGE_FILTER_TYPE";
FilterRankDialog.COMMAND_ID_LOAD_FILTER_VALUES = "CMD_LOAD_FILTER_VALUES";
FilterRankDialog.COMMAND_ID_LOAD_SEARCH_RESULTS = "CMD_LOAD_SEARCH_RESULTS";

FilterRankDialog.prototype.getValueChooserManagerInstances = function()
{
	return this._valueChooserManagerInstances;
}
FilterRankDialog.prototype.setValueChooserManagerInstances = function(vcm)
{
	this._valueChooserManagerInstances = vcm;
}

FilterRankDialog.prototype.isFormattedUnformattedOptionAvailable = function()
{
    return this._isFormattedUnformattedOptionAvailable;
}
FilterRankDialog.prototype.setFormattedUnformattedOptionAvailable = function(isAvailable)
{
    this._isFormattedUnformattedOptionAvailable = isAvailable;
}
//returns true if a formatted comparison is allowed for the
//data item currently being filtered
FilterRankDialog.prototype.isFormattedComparisonAllowed = function()
{
    return this._isFormattedComparisonAllowed;
}
FilterRankDialog.prototype.setFormattedComparisonAllowed = function(isAllowed)
{
    this._isFormattedComparisonAllowed = isAllowed;
}
//returns true if an unformatted comparison is allowed for 
//the data item currently being filtered
FilterRankDialog.prototype.isUnformattedComparisonAllowed = function()
{
    return this._isUnformattedComparisonAllowed;
}
FilterRankDialog.prototype.setUnformattedComparisonAllowed = function(isAllowed)
{
    this._isUnformattedComparisonAllowed = isAllowed;
}
FilterRankDialog.prototype.isFilterTypeCharged = function(filterTypeAsStr)
{
    if (this._filterTypeChargeStatuses == null){
        return false;
    }
    var status = this._filterTypeChargeStatuses[filterTypeAsStr];
    if (status == null) {
        return false;
    }
    return status;
}
FilterRankDialog.prototype.setFilterTypeChargeStatus = function(filterTypeAsStr, isCharged)
{
    if (this._filterTypeChargeStatuses == null) {
        this._filterTypeChargeStatuses = new Array();
    }
    this._filterTypeChargeStatuses[filterTypeAsStr] = isCharged;
}
FilterRankDialog.prototype.scrollSelectedDataItemIntoView = function()
{
    var objForm = this.getForm();
    var selectedIndexInput = objForm.elements['newSelectedDataItemIndex'];
    if (selectedIndexInput == null) {
        return;
    }
    var selectedDiTableRow = findDOM("tr_selectDataItem_"+selectedIndexInput.value);
    if( selectedDiTableRow != null ) { dojo.html.scrollIntoView(selectedDiTableRow); }
	var selectedDiTableCell = findDOM("tr_selectDataItemSpan_"+selectedIndexInput.value);
	if (selectedDiTableCell != null ) { selectedDiTableCell.focus(); }
}
FilterRankDialog.prototype.onDialogPageLoaded = function()
{
    this.scrollSelectedDataItemIntoView();

    var activeControlContainer = this.getControlContainerForActiveFilterType();
    if ( activeControlContainer != null ) 
    {
	    wrsShowElement(activeControlContainer, true);
	    if ( activeControlContainer.onShow != null && typeof activeControlContainer.onShow == "function" )
	    { 
	        activeControlContainer.onShow(true); 
	    }
    }
    
    //ensure upper left corner is appropriately visible on page loads
    window.scroll(0, 0);
  	this.enableLoadButtons();
}

FilterRankDialog.prototype.enableLoadButtons = function()
{
    var objForm = this.getForm();
    var formattedFilterCb = objForm.elements["formattedComparisonCheckbox"];
    var valueTypeFormatted = formattedFilterCb.checked;
    if (valueTypeFormatted) 
    {
       if (this._needFormattedValueLoad && !this._dataItemTypeInOnly)
       {
           // enable formatted buttons
           if (findDOM('formattedValueSelectorForInExpression_loadValuesButton') != null)
			setButtonState(null, 'formattedValueSelectorForInExpression_loadValuesButton', setButtonState.ENABLED);
       }
       else
       {
          // disable formatted buttons
        	if (findDOM('formattedValueSelectorForInExpression_loadValuesButton') != null)
        	  setButtonState(null, 'formattedValueSelectorForInExpression_loadValuesButton', setButtonState.DISABLED);
       }
    }
    if (!valueTypeFormatted)
    { 
       if (this._needUnformattedValueLoad && !this._dataItemTypeInOnly)
       {
	      // enable unformatted buttons
          if (findDOM('unformattedValueSelectorForInExpression_loadValuesButton') != null)
        	 setButtonState(null, 'unformattedValueSelectorForInExpression_loadValuesButton', setButtonState.ENABLED);
    	}
       else
       {
    	   // disable unformatted buttons
           if (findDOM('unformattedValueSelectorForInExpression_loadValuesButton') != null)
        	 setButtonState(null, 'unformattedValueSelectorForInExpression_loadValuesButton', setButtonState.DISABLED);
       }
    }
}

FilterRankDialog.prototype.getForm = function()
{
    return this._masterFormObj;
}
FilterRankDialog.prototype.putFieldSet = function(fieldSetKey, fieldSet)
{
    this._fieldSetMap[fieldSetKey] = fieldSet;
}
FilterRankDialog.prototype.getFieldSet = function(fieldSetKey)
{
    return this._fieldSetMap[fieldSetKey];
}
FilterRankDialog.prototype.getActivePanel = function()
{
    var objForm = this.getForm();
     //TODO make instance based
    var selectedFilterTypeId = "";
    var radioGrp = objForm.elements["selectedFilterType"];
    for (var i=0; i<radioGrp.length; i++) {
    	if (radioGrp[i].checked) {
    		selectedFilterTypeId = radioGrp[i].value;
    	}
    }
    var activePanel = FilterRankDialog.NONE_PANEL;
    if (selectedFilterTypeId == FilterRankDialog.FILTER_NONE) return activePanel;
    if (this._dataItemType == "MEASURE") {
    	if (selectedFilterTypeId == FilterRankDialog.FILTER_FILTER) {
    		activePanel = FilterRankDialog.MEASURE_DATE_PANEL;
    	}
    	else if (selectedFilterTypeId == FilterRankDialog.FILTER_RANK) {
    		activePanel = FilterRankDialog.RANK_PANEL;
    	}
    }
    else if (this._dataItemType == "DATE") {
	    var formattedComparison = false;
	    var formattedComparisonCb = objForm.elements["filterDefinitionForm.formattedComparison"];
	    if (formattedComparisonCb != null) {
	   		formattedComparison = formattedComparisonCb.checked;
	    }
		
    	if (formattedComparison) {
    		activePanel = FilterRankDialog.IN_PANEL;
    	}
    	else {
    		activePanel = FilterRankDialog.MEASURE_DATE_PANEL;
    	}
    }
	else { // this._dataItemType == "CATEGORY"
		activePanel = FilterRankDialog.IN_PANEL;
	}
	return activePanel;
}
FilterRankDialog.prototype.getControlContainerForActiveFilterType = function()
{
    var activeContainer = findDOM(this.getActivePanel());
    return activeContainer;
}

FilterRankDialog.prototype.updateIgnoreCaseCheckBox = function()
{
	var show = ( this.getActivePanel() == FilterRankDialog.IN_PANEL );
	if (show == true) {
		show = (this.isFormattedUnformattedOptionAvailable()) ? true : false;
    }
         
	wrsShowElement( findDOM('ignoreCaseRow'), show);
}

// ------------------------------------------------------------------------
// Handler invoked when the user changes the selected data item (category
// or measure).
// ------------------------------------------------------------------------
FilterRankDialog.prototype.onSelectedDataItemChange = function(newDataItemId, idInputFieldName)
{
    if ( !g_OkToSubmitFilterAndRankForm )
    {
        return;
    }

    var objForm = this.getForm();
    var activeContainer = this.getControlContainerForActiveFilterType();
    if ( !preFilterCommitValidation(objForm, activeContainer) )
    {
        return;
    }

    //Prepare active filter type container for submission and any previously 
    //selected filter type containers for submission. Previously selected
    //filter type containers are prepared so as to remember the settings on 
    //a redraw (e.g. in response to mid-tier validation error) and upon
    //a return to that filter type in the same DATA ITEM dlg session.
    //
    //NOTE: As desired this check includes the requested filter type since
    //it is assumed to be charged by definition.
    this.prepareChargedFilterTypesForSubmission();

    var selectedIdInput = objForm.elements[idInputFieldName];
    var prevSelectedId = selectedIdInput.value;
    selectedIdInput.value = newDataItemId;
    var prevSelectedTableRow = findDOM("tr_selectDataItem_"+prevSelectedId);
    var newSelectedTableRow = findDOM("tr_selectDataItem_"+newDataItemId);
    var prevSelectedTableRowCells = prevSelectedTableRow.cells;
    var newSelectedTableRowCells = newSelectedTableRow.cells;
    var selectedRowColClass = prevSelectedTableRowCells[0].className;

    for( var i=0; i<prevSelectedTableRowCells.length; i++ )
    {
      prevSelectedTableRowCells[i].className = "defaultDataItemCell";
      newSelectedTableRowCells[i].className = selectedRowColClass;
    }
	this.setValueChooserManagerInstances(new Array());

    //identify this as a request to change the selected data item 
    objForm.elements["commandId"].value = FilterRankDialog.COMMAND_ID_CHANGE_DATA_ITEM_SELECTION; 
    var ogAction = objForm.action;
    objForm.action = this._applyActionHandlerPath;
    //targeted action should persist any changes to prev selected item & init form
    //bean with filter information of new selected item
    submitFilterAndRankDlgForm(objForm);
    objForm.action = ogAction; //reset back to original
}

// ------------------------------------------------------------------------
// Handler to be invoked when the user changes the selected filter type
// ------------------------------------------------------------------------
FilterRankDialog.prototype.onFilterTypeChange = function()
{
    // 1)extract the js objects we need to work with
    var objForm = this.getForm();
    var requestedFilterTypeId = "";
    var radioGrp = objForm.elements["selectedFilterType"];
    for (var i=0; i<radioGrp.length; i++) {
    	if (radioGrp[i].checked) {
    		requestedFilterTypeId = radioGrp[i].value;
    		break;
    	}
    }
    var isRequestedFilterTypeCharged = this.isFilterTypeCharged(requestedFilterTypeId);
    if (!isRequestedFilterTypeCharged)
    {
        //Prepare previously selected filter type containers for 
        //submission so as to remember the settings on the the redraw and upon
        //a return to that filter type in the same DATA ITEM dlg session.
        //
        //NOTE: As desired this check skips the requested filter type as well
        this.prepareChargedFilterTypesForSubmission();
        
        //update field that indicates the command to be performed ...
        objForm.elements["commandId"].value = FilterRankDialog.COMMAND_ID_CHANGE_FILTER_TYPE; 
  
        //update field that indicates the request is to be processed by APPLY ...
        var ogAction = objForm.action;
        objForm.action = this._applyActionHandlerPath;
        
        //submit form 
        submitFilterAndRankDlgForm(objForm);
        objForm.action = ogAction; //reset back to original       
        
        return;
    }
    
    // 2) change the filter editor container
    this.switchPanels();
}

FilterRankDialog.prototype.switchPanels = function()
{
    var prevSelectedOpt = this._previousSelectedFilterTypeOption;
    var prevActiveContainer = findDOM(prevSelectedOpt);
    var newActiveContainer  = this.getControlContainerForActiveFilterType();
    if( prevActiveContainer != null )
    {   //purposely hide previous editor container b4 displaying new
        wrsShowElement(prevActiveContainer, false);
        if ( prevActiveContainer.onShow != null && typeof prevActiveContainer.onShow == "function" )
        { prevActiveContainer.onShow(false); }
    }
    if ( newActiveContainer != null ) 
    {
	    wrsShowElement(newActiveContainer, true);
	    if ( newActiveContainer.onShow != null && typeof newActiveContainer.onShow == "function" )
	    { newActiveContainer.onShow(true); }
    }
    this._previousSelectedFilterTypeOption = this.getActivePanel();
}

FilterRankDialog.prototype.prepareChargedFilterTypesForSubmission = function()
{
    // 1)extract the js objects we need to work with
    var objForm = this.getForm();
        //TODO check for IN_PANEL
    if (this.isFilterTypeCharged(2)) // IN_PANEL filter type id
    {
        var ftContainer = findDOM(FilterRankDialog.IN_PANEL);
        if (ftContainer != null && typeof ftContainer.prepareForSubmit == "function") 
        {
            ftContainer.prepareForSubmit(objForm);
        }
    } 
}

FilterRankDialog.prototype.onFormattedComparisonCheckboxChange = function()
{
    if (this._dataItemType == "DATE") {
    	processFormattedComparisonState_forDates();
    }
    else {
		processFormattedComparisonState_selectedValuesForInExpression();
	}
    this.enableLoadButtons();
}

FilterRankDialog.prototype.loadFilterValues = function(isFormattedComparisonRequest)
{
    var objForm = this.getForm();
    
    //Prepare previously selected filter type containers for 
    //submission so as to remember the settings on the the redraw and upon
    //a return to that filter type in the same DATA ITEM dlg session.
    //
    //NOTE: As desired this check skips the requested filter type as well
    this.prepareChargedFilterTypesForSubmission();
        
    //update field that indicates the command to be performed ...
    objForm.elements["commandId"].value = FilterRankDialog.COMMAND_ID_LOAD_FILTER_VALUES; 
  
    //update field that indicates the request is to be processed by APPLY ...
    var ogAction = objForm.action;
    objForm.action = this._applyActionHandlerPath;
        
    //submit form 
    submitFilterAndRankDlgForm(objForm);
    objForm.action = ogAction; //reset back to original       
        
    return true;
}

FilterRankDialog.prototype.loadSearchResults = function()
{
    var objForm = this.getForm();
    this.prepareChargedFilterTypesForSubmission();
    objForm.elements["commandId"].value = FilterRankDialog.COMMAND_ID_LOAD_SEARCH_RESULTS;
	if (objForm.elements["filterDefinitionForm.textToMatch"].value.length < 1) return false;	
    //update field that indicates the request is to be processed by APPLY ...
    var ogAction = objForm.action;
    objForm.action = this._applyActionHandlerPath;
        
    //submit form 
    submitFilterAndRankDlgForm(objForm);
    objForm.action = ogAction; //reset back to original       
        
    return true;
}

FilterRankDialog.prototype.submitAndScroll = function(actionUrl) 
{
    var objForm = this.getForm();
    prepareForFormSubmit_matchValues(objForm); // Fixes 7610960892/S0949051
    var savedAction = objForm.action;
    objForm.action = actionUrl;
    submitFilterAndRankDlgForm(objForm);
    objForm.action = savedAction;
}
  

// ------------------------------------------------------------------------
// Handler invoked when the user changes the selected operator
// Switches between displaying the controls for the "between"
// operator and the controls for the other operators
// ------------------------------------------------------------------------
FilterRankDialog.prototype.onOperatorChange = function()
{
    var objForm = this.getForm();
    var operatorSelectionField = objForm.elements["filterDefinitionForm.operator"];
    var selectedIndex = operatorSelectionField.selectedIndex;
    var operatorChoices = operatorSelectionField.options;
    var newOperatorKey = operatorChoices[selectedIndex].value;
    var fieldSetToActivate = this.getFieldSet(newOperatorKey);
    
    for (var n=0; n<operatorChoices.length; n++)
    {
        if (n != selectedIndex)
        {
            var operatorChoice = operatorChoices[n];
            var choiceFieldSet = this.getFieldSet(operatorChoice.value);
		    if (choiceFieldSet != null 
		        && choiceFieldSet != fieldSetToActivate //some ops may share same field set
		        ) 
		    {
		        choiceFieldSet.deactivate();
		        //choiceFieldSet.show(false);
		    }
        }
    }
    
    //make "new" active visible last for better visual effect
    if (fieldSetToActivate != null) 
    {
        fieldSetToActivate.activate();
    }
}

FilterRankDialog.prototype.FIELDSETKEY_DATE_COMPARISON_VALUE_CONTROLS 
    = "FIELDSETKEY_DATE_COMPARISON_VALUE_CONTROLS";
FilterRankDialog.prototype.FIELDSETKEY_MIN_DATE_CONTROLS 
    = "FIELDSETKEY_MIN_DATE_CONTROLS";
FilterRankDialog.prototype.FIELDSETKEY_MAX_DATE_CONTROLS 
    = "FIELDSETKEY_MAX_DATE_CONTROLS";

function FilterRankDialog() 
{
    this._masterFormObj = null;  
    this._isUnformattedComparisonAllowed = false;
    this._isFormattedComparisonAllowed = false;  
    this._fieldSetMap = new Array();
    this._dataItemType = "";
	this._valueChooserManagerInstances = new Array();
}

//Javascript object that is used to model the dialog and its information.
//Note:
// 
// 1) This object does not represent the window but instead a logical
//    representation of the dialog.
// 2) Supports at most one dialog per window
//
var g_filterRankDialog = new FilterRankDialog(); 

//boolean that indicates whether OK to submit form. Set this flag immediately
//after form is submitted to prevent any subsequent submits while the submit is
//being processed
g_OkToSubmitFilterAndRankForm = true;

//boolean that indicates whether the submit was actually triggered
//from a legal context. The value of this boolean is set programattically
//so that submits are not allowed when for example the user presses ENTER
//in a text field (e.g. the year field). There should be an onsubmit handler
//registered on the form that returns this false when this flag is not set.
g_isLegalSubmitContext = false;

function onFilterRankFormTagSubmit(event)
{
    //alert("vetoing submit := " + !g_isLegalSubmitContext);
    //veto the submit if not a legal form submit
    if (!g_isLegalSubmitContext)
    {
        if(WRSEventUtil.preventDefaultBehavior != null)
        {
            WRSEventUtil.preventDefaultBehavior(event);
        }
    }
    return g_isLegalSubmitContext;
}

function onFormattedComparisonCheckboxChange()
{
    var activeContainer = g_filterRankDialog.getControlContainerForActiveFilterType();
    if (activeContainer != null 
        && activeContainer.processFormattedComparisonState != null)
    {
        activeContainer.processFormattedComparisonState();
    }
}

function enableFormatCheckbox(enable)
{
    var formatCbLabel = findDOM("formattedComparisonCheckboxLabel");
    if (formatCbLabel == null)
        return;
    var formatCb = findDOM("formattedComparisonCheckbox");
    if (formatCb == null)
        return;
        
    formatCbLabel.className = enable ? "label" : "labeldisabled";
    formatCb.disabled = !enable;    
}

function processFormattedComparisonState_forDates()
{
    var objForm = g_filterRankDialog.getForm();

    var formattedComparisonCb = objForm.elements["filterDefinitionForm.formattedComparison"];
    if (formattedComparisonCb == null) {
        return;
    }
	g_filterRankDialog.switchPanels();
	if (g_filterRankDialog._previousSelectedFilterTypeOption == FilterRankDialog.IN_PANEL) {
		// change operator to equal to prevent making a bad filter - S0398886
	    var operatorSelectionField = objForm.elements["filterDefinitionForm.operator"];
	    var operatorKey = operatorSelectionField.options[operatorSelectionField.selectedIndex].value;
	    if (operatorKey != FilterRankDialog.OPERATOR_EQUAL) {
	    	for (var i=0; i<operatorSelectionField.options.length; i++) {
	    		if (operatorSelectionField.options[i].value == FilterRankDialog.OPERATOR_EQUAL) {
	    			operatorSelectionField.selectedIndex = i;
	    			break;
	    		}
	    	}
	    }
	}
}

function processFormattedComparisonState_selectedValuesForInExpression()
{
    var objForm = g_filterRankDialog.getForm();

    var formattedComparisonCb = objForm.elements["filterDefinitionForm.formattedComparison"];
    if (formattedComparisonCb == null) {
        return;
    }

   	var valueTypeFormatted = formattedComparisonCb.checked;
	var unformattedSearchChoice = findDOM("unformattedValueSelectorForInExpression_searchChoice");
	var unformattedSearchValue = findDOM("unformattedValueSelectorForInExpression_searchValue");
	var formattedSearchChoice = findDOM("formattedValueSelectorForInExpression_searchChoice");
	var formattedSearchValue = findDOM("formattedValueSelectorForInExpression_searchValue");
	if (valueTypeFormatted &&
		formattedSearchChoice != null && formattedSearchValue != null) {
		  formattedSearchChoice.value = EditFilterForm_SEARCH_OPERATOR_CONTAINS;
		  formattedSearchValue.value = '';
	}
	else if (unformattedSearchChoice != null && unformattedSearchValue != null) {
		unformattedSearchChoice.value = EditFilterForm_SEARCH_OPERATOR_CONTAINS;
		unformattedSearchValue.value = '';
	}
    updateValueSelectorVisibilityForInExpression();
}

function updateValueSelectorVisibilityForInExpression()
{
    var objForm = g_filterRankDialog.getForm();

    var formattedComparisonCb = objForm.elements["filterDefinitionForm.formattedComparison"];
    if (formattedComparisonCb == null) {
        return;
    }
   	var valueTypeFormatted = formattedComparisonCb.checked;
    wrsShowElement(findDOM("formattedValueSelectorForInExpression_Container"), valueTypeFormatted);
    wrsShowElement(findDOM("unformattedValueSelectorForInExpression_Container"), !valueTypeFormatted);
}

function onShowValueSelectionControlsForInExpression(show)
{
    if (show)
    {
        updateValueSelectorVisibilityForInExpression('filterRankDialogForm');
    }
    updateFormatCheckbox(show);
}

function onShowFilterTypeNoneControls(show)
{
	updateFormatCheckbox(!show);
}

function onShowFilterTypeMeasureOrDateControls(show)
{
	if (show) {
		g_filterRankDialog.onOperatorChange();
	}
	if (g_filterRankDialog._dataItemType == "DATE") {
		updateFormatCheckbox(show);
	}
}

function updateFormatCheckbox(show)
{
    var formatCbContainer = findDOM("formattedComparisonOptionsContainer")
    var containerVisible = formatCbContainer != null 
            && formatCbContainer.style.display != "none";
    if (containerVisible
        && g_filterRankDialog.isFormattedComparisonAllowed()
        && g_filterRankDialog.isUnformattedComparisonAllowed())
    {
        enableFormatCheckbox(show);
    }
    g_filterRankDialog.updateIgnoreCaseCheckBox();
}


// ------------------------------------------------------------------------
// Handler invoked when the user changes the order of the ranking using
// either the "top" or "bottom" labeled radio
// ------------------------------------------------------------------------
function onRankingOrderChange(topRankingRadioId, bottomRankingRadioId)
{
    //alert("changing ranking type");
    var topRankingControls = onRankingOrderChange.topRankingControls;
    var bottomRankingControls = onRankingOrderChange.bottomRankingControls;
    var topRankingRadio = findDOM(topRankingRadioId);
    var bottomRankingRadio = findDOM(bottomRankingRadioId);

    var disableTopControls = !topRankingRadio.checked;
    for( var i=0; i<topRankingControls.length; i++ )
    {   //for now safe to assume topRankingControls.length == bottomRankingControls.length
        var topRankCtrl = topRankingControls[i];
        var bottomRankCtrl = bottomRankingControls[i]
        if (topRankCtrl.enabledStateModifiable != null && topRankCtrl.enabledStateModifiable) {
            if (topRankCtrl.tagName != null && "LABEL" == topRankCtrl.tagName)
            {
            	if (topRankCtrl.className.indexOf('label') == 0) { // className starts with label
            		topRankCtrl.className = (disableTopControls) ? 'labeldisabled' : 'label';
            	}
                //switch className based on ctrl state
            }
            else {
            	topRankCtrl.disabled = disableTopControls;
            }
        }
        if (bottomRankCtrl.enabledStateModifiable != null && bottomRankCtrl.enabledStateModifiable) {
            if (bottomRankCtrl.tagName != null && "LABEL" == bottomRankCtrl.tagName)
            {
            	if (bottomRankCtrl.className.indexOf('label') == 0) { // className starts with label
            		bottomRankCtrl.className = (!disableTopControls) ? 'labeldisabled' : 'label';
            	}
                //switch className based on ctrl state
            }
            else {
          		bottomRankCtrl.disabled = !disableTopControls;
          	}
        }
    }
}

// =============================================================================
//  WrsDateValueCategoryFieldSet -- see citation_fieldSet.js
//
//  Maintains the logic needed to update the state of fields used to specify
//  what type of date value the user wishes to specify (EXPLICIT or RELATIVE).
// =============================================================================
WRSDateValueCategoryFieldSet.prototype = new WRSFieldSet();

function WRSDateValueCategoryFieldSet(formName, categorySelectionRadioNames)
{
    this._formName = formName;
    this._categorySelectionRadioNames = categorySelectionRadioNames;
}

WRSDateValueCategoryFieldSet.prototype.activate = function()
{
    var objForm = document.forms[this._formName];
    var dvcRadios = objForm.elements[this._categorySelectionRadioNames];
    var radioCount = dvcRadios.length;
    
    var newCategoryRadio = null;
    for (var radioIndex=0; radioIndex<radioCount; radioIndex++)
    {
        var radio = dvcRadios[radioIndex];
        if (radio.checked)
        {
            newCategoryRadio = radio;
        }
        else
        {
            var radioFieldSet = g_filterRankDialog.getFieldSet(radio.id);
            if (radioFieldSet != null && radioFieldSet.deactivate != null)
            {
                radioFieldSet.deactivate();
            }
        }
    }
    
    if (newCategoryRadio != null)
    {
        var newRadioFieldSet = g_filterRankDialog.getFieldSet(newCategoryRadio.id);
        if (newRadioFieldSet != null && newRadioFieldSet.activate != null)
        {
            newRadioFieldSet.activate();
        }
    }
}

WRSDateValueCategoryFieldSet.prototype.deactivate = function()
{
    //IMPORTANT: A NOOP for this field set. This is not default behavior.
}

//@param fieldSetKey key that should be used to lookup the field set associated with
//       "valueCategory" selection radios where EXPLICIT and RELATIVE are the category types.
function onDateValueCategoryChange(fieldSetKey)
{
    g_filterRankDialog.getFieldSet(fieldSetKey).activate();
    
//    g_filterRankDialog.getFieldSet(g_filterRankDialog.FIELD_SET_KEY_COMPARISON_DATE_VALUE_CATEGORY_RADIOS)
//        .activate();
}

// =============================================================================
//  WrsRelativeDateValueFieldSet -- see citation_fieldSet.js
//
//  Maintains the logic needed to update the state of fields used to specify
//  relative date values when creating a comparison or between filter 
// =============================================================================
WRSRelativeDateValueFieldSet.prototype = new WRSFieldSet();

function WRSRelativeDateValueFieldSet(formName, relativeDateDropdownId)
{
    this._formName = formName;
    this._activeFieldSet = null;
    this._relativeDateDropdownId = relativeDateDropdownId;
}

WRSRelativeDateValueFieldSet.prototype.activate = function()
{
    var relativeValueAliasDropdown = findDOM(this._relativeDateDropdownId);
    relativeValueAliasDropdown.disabled = false;
    relativeValueAliasDropdown.size = 5;

    //determine the fieldset associated with the alias so that it can be activated.
    var aliasOptions = relativeValueAliasDropdown.options;
    var optionCount = aliasOptions.length;
    var selectedAliasOption = aliasOptions[relativeValueAliasDropdown.selectedIndex];
    var targetFieldSetKey = selectedAliasOption.value;
    var targetAliasFieldSet = this.getEntry(targetFieldSetKey);
    
    for (var optionIndex=0; optionIndex<optionCount; optionIndex++)
    {
        var aliasOption = aliasOptions[optionIndex];
        var aliasFieldSet = this.getEntry(aliasOption.value);
        if (aliasFieldSet != targetAliasFieldSet
            && aliasFieldSet != null 
            && aliasFieldSet.deactivate != null)
        {
            aliasFieldSet.deactivate();
        }
    }
    if (targetAliasFieldSet != null && targetAliasFieldSet.activate != null)
    {
        targetAliasFieldSet.activate();
    }
    this._activeFieldSet = targetAliasFieldSet;
}

WRSRelativeDateValueFieldSet.prototype.deactivate = function()
{
    //var objForm = document.forms[this._formName];
    var relativeValueAliasDropdown = findDOM(this._relativeDateDropdownId);
    relativeValueAliasDropdown.disabled = true;
    relativeValueAliasDropdown.size = 1;
    
    //Deactivate the fieldset associated with the alias.
    //This involves deactivating the sub field set (if any) associated with
    //the aliases in the dropdown. Note that in the majority of cases, the 
    //deactivate() calls will only do work on the currently selected option
    //since the changing the selected option should deactivate the previously
    //selected option. The initial drawing of the page may deactivate all
    //the fieldsets however.
    var aliasOptions = relativeValueAliasDropdown.options;
    var optionCount = aliasOptions.length;
    for (var optionIndex=0; optionIndex<optionCount; optionIndex++)
    {
        var aliasOption = aliasOptions[optionIndex];
        var aliasFieldSet = this.getEntry(aliasOption.value);
        if (aliasFieldSet != null && aliasFieldSet.deactivate != null)
        {
            aliasFieldSet.deactivate();
        }
    }
}

function onRelativeDateValueTypeChange(relativeValueFieldSetKey)
{
    g_filterRankDialog.getFieldSet(relativeValueFieldSetKey)
        .activate();
}

function preFilterCommitValidation(objForm, activeContainer)
{
    //ensure that active container has valid info b4 submitting form
    var invalidFormElement = activeContainer.validate(objForm);
    if( invalidFormElement != null )
    {
    	if (activeContainer.validate.validationErrorMessage.length > 0) {
	        alert(activeContainer.validate.validationErrorMessage);
	    }
        if (invalidFormElement.focus != null) {
	        invalidFormElement.focus();
	    }
        return false;
    }
    return true;
}


// ------------------------------------------------------------------------
// Handler invoked when the user submits the form by pressing the ok
// button
// ------------------------------------------------------------------------
function onCommitRankFilter(formName, cmdKeyInputFieldName, commitCmdId)
{
    var objForm = document.forms[formName];
    var activeContainer = g_filterRankDialog.getControlContainerForActiveFilterType();
    if ( !preFilterCommitValidation(objForm, activeContainer) )
    {
        return;
    }

    //Prepare active filter type container for submission and any previously 
    //selected filter type containers for submission. Previousl selected
    //filter type containers are prepared so as to remember the settings on 
    //a redraw (e.g. in response to mid-tier validation error) and upon
    //a return to that filter type in the same DATA ITEM dlg session.
    //
    //NOTE: As desired this check includes the requested filter type since
    //it is assumed to be charged by definition.
    g_filterRankDialog.prepareChargedFilterTypesForSubmission();

    //update the value of field that tells the mid-tier what cmd to execute
    var cmdKeyInputField = objForm.elements[cmdKeyInputFieldName];
    if( cmdKeyInputField != null )
    {
        cmdKeyInputField.value = commitCmdId;
    }
    submitFilterAndRankDlgForm(objForm);
}

//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
// Submits the dialogs form and performs any post submit activities. All submits
// of the corresponding dialog should be done using this method. This means
// care must be taken to disable the default submit actions of certain fields
// (e.g. textEntrys) contained within the form.
//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
function submitFilterAndRankDlgForm(objForm)
{
    g_isLegalSubmitContext = true; //all contexts that submit using this method are legal
    if (g_OkToSubmitFilterAndRankForm)
    {
        //submit the form, notifying any listeners of the pending event beforehand
        eventNotify(objForm,'submit',false);
        objForm.submit();
        afterFilterAndRankDlgSubmit(objForm)
    }
}

//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
// Should be invoked after submit is called. This function displays the
// wait cursor within the dialog and updates the form so that no subsequent form
// submits can occur.
//
// WARNING: It is essential that this be called after submit and not before.
// This is b/c this method may result in form fields being disabled (maybe
// indirectly - the parent container is disabled) and the field values
// will not be submitted if disabled prior to the submit call.
//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
function afterFilterAndRankDlgSubmit(objForm)
{
    g_OkToSubmitFilterAndRankForm = false;
    //change to wait cursor since all submissions of this form result in a page reload.
    if (setDocumentCursorStyle != null) {
        setDocumentCursorStyle("wait");
    }

    //disable OK, CANCEL, HELP buttons which are outside the form
    if( setButtonState != null ) {
        setButtonState(null /*event*/, "rankFilterSubmitButton", setButtonState.DISABLED );
        setButtonState(null /*event*/, "rankFilterCancelButton", setButtonState.DISABLED );
        setButtonState(null /*event*/, "helpButton", setButtonState.DISABLED );
    }

    //disable data item selection table. Some user agents might not honor this
    //so as added precaution set a flag that indicates subsequent submissions of
    //of form are not allowed. NOTE: use table not scrolling div. Weird alignment
    //issues occur when disabling the div.
    //div id ="rankAndFilterDataItemSelectionContainer"
    var diSelectContainer = findDOM("rankAndFilterDataItemSelectionTableBody");
    if (diSelectContainer != null)
    {
       diSelectContainer.disabled = true;
    }
 
    setFormElementsEnabled(objForm, false);
}

function setFormElementsEnabled(objForm, enabled)
{
    var formEls = objForm.elements;
    var formElCount = formEls.length;
    for( var i=0; i<formElCount; i++ )
    {
        formEls[i].disabled = !enabled;
    }
}

/*
XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
  Methods that perform the pre processing for particular filter types before
  submission of the form data for a particular filter definition.
XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
 */

function prepareForFormSubmit_matchValues(objForm)
{
    _prepareMatchValueSettingsForSubmission(objForm, false);
    _prepareMatchValueSettingsForSubmission(objForm, true);
    var formattedComparisonCb = objForm.elements["filterDefinitionForm.formattedComparison"];
    if (formattedComparisonCb == null) {
        return;
    }

   	var valueTypeFormatted = formattedComparisonCb.checked;
   	if (valueTypeFormatted) {
    	var formattedSearchChoice = findDOM("formattedValueSelectorForInExpression_searchChoice");
    	var formattedSearchValue = findDOM("formattedValueSelectorForInExpression_searchValue");
    	if (formattedSearchChoice != null && formattedSearchValue != null) {
    		objForm.elements["filterDefinitionForm.searchOperator"].value = formattedSearchChoice.value;
    		objForm.elements["filterDefinitionForm.textToMatch"].value = formattedSearchValue.value;	
    	}
   	}
   	else {
    	var unformattedSearchChoice = findDOM("unformattedValueSelectorForInExpression_searchChoice");
    	var unformattedSearchValue = findDOM("unformattedValueSelectorForInExpression_searchValue");
		if (unformattedSearchChoice != null && unformattedSearchValue != null) {
			objForm.elements["filterDefinitionForm.searchOperator"].value = unformattedSearchChoice.value;
			objForm.elements["filterDefinitionForm.textToMatch"].value = unformattedSearchValue.value;
		}
   	}
    
}// end prepareForFormSubmit_matchValues() method definition

function _prepareMatchValueSettingsForSubmission(objForm, isFormattedValueSettings)
{
    //alert("preparing " + (isFormattedValueSettings?"formatted":"unformatted") + " values");
    var valueFieldsName = null;
    var valueSelectionWidget = null; //may still be null after attempted assignment
    
    if (isFormattedValueSettings)
    {
        valueFieldsName = "filterDefinitionForm.selectedFormattedValues";
        valueSelectionWidget = objForm.selectedFormattedValuesWidgetForInExpression;
    }
    else
    {
        valueFieldsName = "filterDefinitionForm.selectedUnformattedValues";
        valueSelectionWidget = objForm.selectedUnformattedValuesWidgetForInExpression;
    }
    
    var prevMatchValueFields = objForm.elements[valueFieldsName];
    if( prevMatchValueFields != null )
    {
        if (prevMatchValueFields.length > 0) {
            var prevItemCount = prevMatchValueFields.length;
            for (var n=0; i<prevItemCount; i++) {
                var prevField = prevMatchValueFields[n];
                //alert("Removing field from document --> name [" + this.idField.name + "] with value { " + this.idField.value + " }") ;
                objForm.removeChild(prevField);
            }
        }
    }
    
    //no-op if no widget for selection present in ui (the option of filtering on both formatted
    //and unformatted values may not be an option or desired)
    if( valueSelectionWidget != null )
    {
        var valueObjs = valueSelectionWidget.options;
        var valueCount = valueObjs.length;
        for( var i=0; i<valueCount; i++ )
        {
            var valueObj = valueObjs[i];
            //alert("adding value index := " + valueObj.value + "; valueLabel := " + valueObj.text );
            var valueField = document.createElement('input');
            valueField.type = 'hidden';
            valueField.name = valueFieldsName;
            valueField.value = valueObj.value;
            objForm.appendChild(valueField);
        }//for
    }//if( valueSelectionWidget != null )
}// end _prepareMatchValueSettingsForSubmission() method definition


/*
XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
  Methods used to validate form field values on edit
XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
 */

function wrs_filterRankUtil()
{

}

wrs_filterRankUtil.whiteSpaceRegExpTest = new RegExp("\\s", "g");

wrs_filterRankUtil._isEmptyOrWhitespace = function(strTest)
{
    //current impl doesn't account for string of whitespace
    var retval = ( strTest == null || strTest.length == 0 );

    if( !retval )
    {   //nonempty so test for all whitespace
        var matchResult = strTest.match(wrs_filterRankUtil.whiteSpaceRegExpTest);
        var isWhiteSpaceOnly = ( matchResult != null && matchResult.length == strTest.length );
        retval = isWhiteSpaceOnly;
    }
    return retval;
}

function validateRankingAmountForField(rankAmountField, isPercent)
{
    var rankAmountValue = rankAmountField.value;
    rankAmountValue = (rankAmountValue.length == 0) ? null : window.parseInt(rankAmountValue, 10 /*base-10*/);
    return validateRankingAmount(rankAmountValue, isPercent);
}

function validateRankingAmount(rankAmount, isPercent)
{
    var retval = validateRankingAmount.VALID;
    //verify a positive base-10 # && (< 100 if isPercent)
    if ( rankAmount == null )
    {
        retval = validateRankingAmount.NO_VALUE_SPECIFIED;
    }
    else if ( ( window.isNaN != null && window.isNaN(rankAmount) )
        || (rankAmount <= 0)
        || (isPercent && rankAmount > 100 ) )
    {
        retval = validateRankingAmount.ILLEGAL_VALUE_SPECIFIED;
    }
    return retval;
}
//Possible return values for validateRankingAmount
validateRankingAmount.VALID = 0;
validateRankingAmount.NO_VALUE_SPECIFIED = 1;
validateRankingAmount.ILLEGAL_VALUE_SPECIFIED = 2;


/* ************************************************************************
  Methods used to validate group boxes on form submission
************************************************************************* */

function validate_filterNone(objForm)
{
    return null;
}

function validate_selectCategoryValues(objForm)
{
	var invalidField = null;
    var formattedFilterCb = objForm.elements["formattedComparisonCheckbox"];
    var valueTypeFormatted = formattedFilterCb.checked;
    var valueSelectionWidget = (valueTypeFormatted) ?
		objForm.selectedFormattedValuesWidgetForInExpression :
		objForm.selectedUnformattedValuesWidgetForInExpression;
	if (valueSelectionWidget.options.length < 1) {
	 	return valueSelectionWidget;
	}
    return null;
}


//TODO needsRDWork
function validate_comparisonFilter(objForm)
{
    var invalidField = null;
    // ---------------------------- 
    var operatorSelectionField = objForm.elements["filterDefinitionForm.operator"];
    var operatorKey = operatorSelectionField.options[operatorSelectionField.selectedIndex].value;
    var activeContainer = g_filterRankDialog.getControlContainerForActiveFilterType();
    if ( FilterRankDialog.OPERATOR_BETWEEN == operatorKey
            || FilterRankDialog.OPERATOR_NOT_BETWEEN == operatorKey )
    {
        //TODO needsRDWork
       if (g_filterRankDialog._dataItemType == "DATE") {
           var startDateRadioFixed = findDOM("betweenDateCategoryRadioExplicitMin");
           if (startDateRadioFixed != null && startDateRadioFixed.checked) {
	       	   var startDateObj = sas_getJavaScriptObject('filterDefinitionForm_startDate_explicitDate_datePicker');
	           if (startDateObj != null) {
				   if (startDateObj.enabled && startDateObj.validate() == false) {
				      	return startDateObj.comboComponent;
				   }
			   }
		   }
		   
		   var endDateRadioFixed = findDOM("betweenDateCategoryRadioExplicitMax");
		   if (endDateRadioFixed != null && endDateRadioFixed.checked) {
	       	   var endDateObj = sas_getJavaScriptObject('filterDefinitionForm_endDate_explicitDate_datePicker');
	           if (endDateObj != null) {
				   if (endDateObj.enabled && endDateObj.validate() == false) {
				      	return endDateObj.comboComponent;
				   }
			   }
		   }
		   
	  }
      return invalidField;
	        
    }
    else if ( FilterRankDialog.OPERATOR_MISSING == operatorKey
            || FilterRankDialog.OPERATOR_NOT_MISSING == operatorKey )
    {
        return invalidField;
    }
    //else fall through
    
    //TODO date vs non-date logic will need to be split differently 
    // if (date)
    // {
    //   if between
    //   ...
    //   else
    // }
    // else
    // ---------------------------- 
    
    var valueField = objForm.elements["filterDefinitionForm.value"];
    if (valueField != null)
    {   // validate measure value
        if( wrs_filterRankUtil._isEmptyOrWhitespace(valueField.value) )
        {
            invalidField = valueField;
            if( validate_comparisonFilter.validationErrorMessage == null )
            {
                validate_comparisonFilter.validationErrorMessage = "A value must be entered!";
            }
        }
    }
    else
    {
       var fixedDateRadio = findDOM("simpleComparisonDateCategoryExplicit");
       if (fixedDateRadio != null && fixedDateRadio.checked) {
	       var explicitDateObj = sas_getJavaScriptObject('filterDefinitionForm_dateValue_explicitDate_datePicker');
	        //validate date fields if necessary
	       if (explicitDateObj != null) {
			   if (explicitDateObj.enabled && explicitDateObj.validate() == false) {
			      	return explicitDateObj.comboComponent;
			   }
		   }
		}
    }

    return invalidField;
}

function validate_rank(objForm)
{
    //alert("validating rank information");
    var retval = null;
    // 1) aquire refs to needed form ctrls
    var isTopRanking = findDOM("rankingOrder_top").checked;
    var rankAmountField = isTopRanking ? findDOM("rankingOrder_top_amount")    : findDOM("rankingOrder_bottom_amount");
    var isPercentField  = isTopRanking ? findDOM("rankingOrder_top_isPercent") : findDOM("rankingOrder_bottom_isPercent");
    // 2) determine if form ctrl values are valid
    var validationCode = validateRankingAmountForField(rankAmountField, isPercentField.checked);
    if ( validationCode != validateRankingAmount.VALID )
    {
        retval = rankAmountField;
        switch (validationCode) {
            case validateRankingAmount.NO_VALUE_SPECIFIED:
                validate_rank.validationErrorMessage = validate_rank.MSG_NO_VALUE_SPECIFIED != null ? validate_rank.MSG_NO_VALUE_SPECIFIED : null;
                break;
            case validateRankingAmount.ILLEGAL_VALUE_SPECIFIED:
                validate_rank.validationErrorMessage = validate_rank.MSG_ILLEGAL_VALUE_SPECIFIED != null ? validate_rank.MSG_ILLEGAL_VALUE_SPECIFIED : null
                break;
        }
        if( validate_rank.validationErrorMessage == null )
        {
            validate_rank.validationErrorMessage = "Please enter a valid rank value.";
        }
    }

    return retval;
}

function setSelectedSearchChoice(selectId, selectedValueId) {
	var select = findDOM(selectId);
	var selectedValue = findDOM(selectedValueId).value;
	var nOptions = select.length;
	var selectedIndex = 0; // default to first one
	for (var i=0; i<select.length; i++) {
		var option = select.options[i];
		if (option.value == selectedValue) {
			selectedIndex = i;
			break;
		}
	}
	select.selectedIndex = selectedIndex;
}
